@@ -3,7 +3,7 @@
  *
  * NOTE: this code illustrates how to make a dpi-program.
  *
- * Copyright 2002-2004 Jorge Arellano Cid <jcid@dillo.org>
+ * Copyright 2002-2006 Jorge Arellano Cid <jcid@dillo.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -34,15 +34,25 @@
 #include <netdb.h>
 #include <fcntl.h>
 #include <signal.h>
+#include "../dpip/dpip.h"
 #include "dpiutil.h"
 
 #include <glib.h>
 
-/* This one os tricky, some sources state it should include the byte
+/* This one is tricky, some sources state it should include the byte
  * for the terminating NULL, and others say it shouldn't. */
 # define D_SUN_LEN(ptr) ((size_t) (((struct sockaddr_un *) 0)->sun_path) \
                         + strlen ((ptr)->sun_path))
 
+#define DOCTYPE \
+   "<!DOCTYPE HTML PUBLIC '-//W3C//DTD HTML 4.01 Transitional//EN'>\n"
+
+/*
+ * Notes on character escaping:
+ *   - Basically things are saved unescaped and escaped when in memory.
+ *   - &<>"' are escaped in titles and sections and saved unescaped.
+ *   - ' is escaped as %27 in URLs and saved escaped.
+ */
 typedef struct {
    gint key;
    gint section;
@@ -81,6 +91,7 @@ static int MODIFY_PAGE_NUM = 1;
 /* -- HTML templates ------------------------------------------------------- */
 
 char *mainpage_header =
+DOCTYPE
 "<html>\n"
 "<head>\n"
 "<title>Bookmarks</title>\n"
@@ -99,6 +110,7 @@ char *mainpage_header =
 "<br>\n";
 
 char *modifypage_header =
+DOCTYPE
 "<html>\n"
 "<head>\n"
 "<title>Bookmarks</title>\n"
@@ -226,6 +238,7 @@ char *modifypage_footer =
 
 /* ------------------------------------------------------------------------- */
 char *modifypage_add_section_page =
+DOCTYPE
 "<html>\n"
 "<head>\n"
 "<title>Bookmarks</title>\n"
@@ -265,6 +278,7 @@ char *modifypage_add_section_page =
 
 /* ------------------------------------------------------------------------- */
 char *modifypage_update_header =
+DOCTYPE
 "<html>\n"
 "<head>\n"
 "<title>Bookmarks</title>\n"
@@ -327,6 +341,7 @@ char *modifypage_update_footer =
 
 /* ------------------------------------------------------------------------- */
 char *modifypage_add_url =
+DOCTYPE
 "<html>\n"
 "<head>\n"
 "<title>Bookmarks</title>\n"
@@ -397,29 +412,6 @@ static char *make_one_line_str(char *str)
    return new_str;
 }
 
-/*
- * Use character stuffing for characters given in 'esc_set'.
- * Return value: New stuffed string.
- */
-static gchar *Stuff_chars(const gchar *str, const gchar *esc_set)
-{
-   gint i;
-   gchar *p, *sstr = NULL;
-
-   if ((p = strpbrk(str, esc_set))) {
-      /* start stuffing */
-      GString *gstr = g_string_new(str);
-      for (i = p - str; gstr->str[i]; ++i)
-         if (strchr(esc_set, gstr->str[i])) {
-            g_string_insert_c(gstr, i, gstr->str[i]);
-            ++i;
-         }
-      sstr = gstr->str;
-      g_string_free(gstr, FALSE);
-   }
-   return (sstr ? sstr : g_strdup(str));
-}
-
 /*
  * Given an urlencoded string, return it to the original version.
  */
@@ -483,13 +475,11 @@ static char *Get_line(FILE *stream)
 static int Bmsrv_dpi_send_status_msg(SockHandler *sh, char *str)
 {
    int st;
-   GString *gstr = g_string_new("");
-   gchar *sstr = Stuff_chars(str, "'\"");
+   char *d_cmd;
 
-   g_string_sprintf(gstr, "<dpi cmd='send_status_message' msg='%s'>", sstr);
-   st = sock_handler_write_str(sh, gstr->str, 1);
-   g_string_free(gstr, TRUE);
-   g_free(sstr);
+   d_cmd = a_Dpip_build_cmd("cmd=%s msg=%s", "send_status_message", str);
+   st = sock_handler_write_str(sh, d_cmd, 1);
+   g_free(d_cmd);
    return st;
 }
 
@@ -552,8 +542,8 @@ static void Bms_add(gint section, char *url, char *title)
    node = g_new(BmRec, 1);
    node->key = ++bm_key;
    node->section = section;
-   node->url = g_strdup(url);
-   node->title = g_strdup(title);
+   node->url = Escape_uri_str(url, "'");
+   node->title = Escape_html_str(title);
    B_bms = g_slist_append(B_bms, node);
 }
 
@@ -566,7 +556,7 @@ static void Bms_sec_add(char *title)
 
    node = g_new(BmSec, 1);
    node->section = sec_key++;
-   node->title = g_strdup(title);
+   node->title = Escape_html_str(title);
    B_secs = g_slist_append(B_secs, node);
 }
 
@@ -643,10 +633,8 @@ static void Bms_update_title(gint key, gchar *n_title)
    list = g_slist_find_custom(B_bms, GINT_TO_POINTER(key), Bms_key_cmp);
    if (list) {
       BmRec *node = list->data;
-      if (strcmp(node->title, n_title)) {
-         g_free(node->title);
-         node->title = g_strdup(n_title);
-      }
+      g_free(node->title);
+      node->title = Escape_html_str(n_title);
    }
 }
 
@@ -661,10 +649,8 @@ static void Bms_update_sec_title(gint key, gchar *n_title)
                               Bms_sec_cmp);
    if (list) {
       BmSec *node = list->data;
-      if (strcmp(node->title, n_title)) {
-         g_free(node->title);
-         node->title = g_strdup(n_title);
-      }
+      g_free(node->title);
+      node->title = Escape_html_str(n_title);
    }
 }
 
@@ -760,7 +746,7 @@ static void Bms_check_import(void)
 static int Bms_load(void)
 {
    FILE *BmTxt;
-   char *buf, *p, *url, *title;
+   char *buf, *p, *url, *title, *u_title;
    int section;
    struct stat TimeStamp;
 
@@ -783,11 +769,12 @@ static int Bms_load(void)
          p = strchr(p, ' '); *p = 0;
          title = ++p;
          p = strchr(p, '\n'); *p = 0;
-         Bms_add(section, url, title);
-         //g_print("Section: %d Url: %s Title: %s\n", section, url, title);
+         u_title = Unescape_html_str(title);
+         Bms_add(section, url, u_title);
+         g_free(u_title);
 
       } else if (buf[0] == ':' && buf[1] == 's') {
-         // section = strtol(buf + 2, NULL, 10);
+         /* section = strtol(buf + 2, NULL, 10); */
          p = strchr(buf + 2, ' ');
          title = ++p;
          p = strchr(p, '\n'); *p = 0;
@@ -829,7 +816,6 @@ static int Bms_cond_load(void)
       Bms_load();
       st = 1;
    }
-   //g_print("\n >>> %s\n\n", st ? "LOADED" : "KEPT");
    return st;
 }
 
@@ -845,8 +831,9 @@ static int Bms_save(void)
    BmRec *bm_node;
    BmSec *sec_node;
    GSList *list, *list2;
-   GString *gstr = g_string_new("");
    struct stat BmStat;
+   gchar *u_title;
+   GString *gstr = g_string_new("");
 
    /* make a safety backup */
    if (stat(BmFile, &BmStat) == 0 && BmStat.st_size > 256) {
@@ -867,8 +854,10 @@ static int Bms_save(void)
    /* save sections */
    for (list = B_secs; list; list = list->next) {
       sec_node = list->data;
-      g_string_sprintf(gstr, ":s%d: %s\n", sec_node->section, sec_node->title);
+      u_title = Unescape_html_str(sec_node->title);
+      g_string_sprintf(gstr, ":s%d: %s\n", sec_node->section, u_title);
       fwrite(gstr->str, (size_t)gstr->len, 1, BmTxt);
+      g_free(u_title);
    }
 
    /* save bookmarks  (section url title) */
@@ -877,9 +866,11 @@ static int Bms_save(void)
       for (list2 = B_bms; list2; list2 = list2->next) {
          bm_node = list2->data;
          if (bm_node->section == sec_node->section) {
+            u_title = Unescape_html_str(bm_node->title);
             g_string_sprintf(gstr, "s%d %s %s\n",
-                             bm_node->section, bm_node->url, bm_node->title);
+                             bm_node->section, bm_node->url, u_title);
             fwrite(gstr->str, (size_t)gstr->len, 1, BmTxt);
+            g_free(u_title);
          }
       }
    }
@@ -901,13 +892,15 @@ static int Bms_save(void)
  */
 static int Bmsrv_add_bm(SockHandler *sh, char *url, char *title)
 {
+   char *u_title;
    char *msg="Added bookmark!";
    int section = 0;
 
-   //g_print("Adding:\n  %s\n  %s\n", title, url);
-
    /* Add in memory */
-   Bms_add(section, url, title);
+   u_title = Unescape_html_str(title);
+   Bms_add(section, url, u_title);
+   g_free(u_title);
+
    /* Write to file */
    Bms_save();
 
@@ -947,14 +940,12 @@ static void Bmsrv_count_urls_and_sections(char *url, gint *n_sec, gint *n_url)
  */
 static int Bmsrv_send_reload_request(SockHandler *sh, char *url)
 {
-   gint st = 0;
-   GString *gstr = g_string_new("");
-
-   g_string_sprintf(gstr, "<dpi cmd='reload_request' url='%s'>", url);
-   if (sock_handler_write_str(sh, gstr->str, 1))
-      st = 1;
-   g_string_free(gstr, TRUE);
+   gint st;
+   char *d_cmd;
 
+   d_cmd = a_Dpip_build_cmd("cmd=%s url=%s", "reload_request", url);
+   st = sock_handler_write_str(sh, d_cmd, 1) ? 1 : 0;
+   g_free(d_cmd);
    return st;
 }
 
@@ -970,8 +961,6 @@ static int Bmsrv_send_modify_page(SockHandler *sh)
    BmSec *sec_node;
    BmRec *bm_node;
 
-   //g_print("  sending page body...\n");
-
    if (!gstr)
       gstr = g_string_new("");
 
@@ -1030,7 +1019,6 @@ static int Bmsrv_send_modify_page(SockHandler *sh)
    if (sock_handler_write_str(sh, modifypage_footer, 1))
       return 1;
 
-   //g_print("  page body sent.\n");
    return 2;
 }
 
@@ -1040,13 +1028,10 @@ static int Bmsrv_send_modify_page(SockHandler *sh)
  */
 static int Bmsrv_send_modify_page_add_section(SockHandler *sh)
 {
-   //g_print("  sending page body...\n");
-
    /* send modify page2 */
    if (sock_handler_write_str(sh, modifypage_add_section_page, 1))
       return 1;
 
-   //g_print("  page body sent.\n");
    return 2;
 }
 
@@ -1146,19 +1131,16 @@ static int Bmsrv_send_modify_update(SockHandler *sh, char *url)
  */
 static int Bmsrv_send_modify_answer(SockHandler *sh, char *url)
 {
-   GString *gstr = g_string_new("");
+   char *d_cmd;
+   int st;
 
-   g_string_sprintf(gstr, "<dpi cmd='start_send_page' url='%s'>", url);
-   //g_print("  cmd_buf=%s\n", gstr->str);
-   /* Send dpi command */
-   if (sock_handler_write_str(sh, gstr->str, 1) != 0) {
-      g_string_free(gstr, TRUE);
+   d_cmd = a_Dpip_build_cmd("cmd=%s url=%s", "start_send_page", url);
+   st = sock_handler_write_str(sh, d_cmd, 1);
+   g_free(d_cmd);
+   if (st != 0)
       return 1;
-   }
-   g_string_free(gstr, TRUE);
 
    /* Send HTTP header */
-   //g_print("  sending Header...\n");
    if (sock_handler_write(sh, Header, strlen(Header), 0) != 0) {
       return 1;
    }
@@ -1354,7 +1336,7 @@ static int Bmsrv_modify_add_section(SockHandler *sh, char *url)
  */
 static int Bmsrv_modify_add_url(SockHandler *sh, char *s_url)
 {
-   char *p, *q, *title, *url;
+   char *p, *q, *title, *u_title, *url;
    int i;
    static gint section = 0;
 
@@ -1383,7 +1365,9 @@ static int Bmsrv_modify_add_url(SockHandler *sh, char *s_url)
    if (strlen(title) && strlen(url)) {
       Unencode_str(title);
       Unencode_str(url);
-      Bms_add(section, url, title);
+      u_title = Unescape_html_str(title);
+      Bms_add(section, url, u_title);
+      g_free(u_title);
    }
    g_free(title);
    g_free(url);
@@ -1612,25 +1596,28 @@ static int send_bm_page(SockHandler *sh)
  */
 static int Bmsrv_parse_buf(SockHandler *sh, char *Buf, size_t BufSize)
 {
-   char *msg1="<dpi cmd='chat' msg='Hi browser'>";
-   char *msg2="<dpi cmd='chat' msg='Is it worth?'>";
-   char *msg3="<dpi cmd='chat' msg='Ok, send it'>";
-
-   char *p, *cmd, *url, *title, *msg, cmd_buf[16*1024];
+   static char *msg1=NULL, *msg2=NULL, *msg3=NULL;
+   char *p, *cmd, *d_cmd, *url, *title, *msg;
    int st;
 
+   if (!msg1) {
+     /* Initialize data for the "chat" command. */
+     msg1 = a_Dpip_build_cmd("cmd=%s msg=%s", "chat", "Hi browser");
+     msg2 = a_Dpip_build_cmd("cmd=%s msg=%s", "chat", "Is it worth?");
+     msg3 = a_Dpip_build_cmd("cmd=%s msg=%s", "chat", "Ok, send it");
+   }
+
    if (!(p = strchr(Buf, '>'))) {
       /* Haven't got a full tag */
       g_print("Haven't got a full tag!\n");
       return 1;
    }
 
-   cmd = Get_attr_value(Buf, BufSize, "cmd");
-   //g_print("cmd: %s\n", cmd ? cmd : "NULL");
+   cmd = a_Dpip_get_attr(Buf, BufSize, "cmd");
 
    if (cmd && strcmp(cmd, "chat") == 0) {
       g_free(cmd);
-      msg = Get_attr_value(Buf, BufSize, "msg");
+      msg = a_Dpip_get_attr(Buf, BufSize, "msg");
       if (*msg == 'H') {
          /* "Hi server" */
          if (sock_handler_write_str(sh, msg1, 1))
@@ -1657,8 +1644,8 @@ static int Bmsrv_parse_buf(SockHandler *sh, char *Buf, size_t BufSize)
 
    } else if (cmd && strcmp(cmd, "add_bookmark") == 0) {
       g_free(cmd);
-      url = Get_attr_value(Buf, BufSize, "url");
-      title = Get_attr_value(Buf, BufSize, "title");
+      url = a_Dpip_get_attr(Buf, BufSize, "url");
+      title = a_Dpip_get_attr(Buf, BufSize, "title");
       if (strlen(title) == 0) {
          g_free(title);
          title = g_strdup("(Untitled)");
@@ -1671,7 +1658,7 @@ static int Bmsrv_parse_buf(SockHandler *sh, char *Buf, size_t BufSize)
 
    } else if (cmd && strcmp(cmd, "open_url") == 0) {
       g_free(cmd);
-      url = Get_attr_value(Buf, BufSize, "url");
+      url = a_Dpip_get_attr(Buf, BufSize, "url");
 
       if (strcmp(url, "dpi:/bm/modify") == 0) {
          st = Bmsrv_send_modify_answer(sh, url);
@@ -1683,29 +1670,27 @@ static int Bmsrv_parse_buf(SockHandler *sh, char *Buf, size_t BufSize)
          return st;
       }
 
-      sprintf(cmd_buf, "<dpi cmd='start_send_page' url='%s'>", url);
-      //g_print("  cmd_buf=%s\n", cmd_buf);
-      /* Send dpi command */
-      if (sock_handler_write(sh, cmd_buf, strlen(cmd_buf), 1) != 0) {
+
+      d_cmd = a_Dpip_build_cmd("cmd=%s url=%s", "start_send_page", url);
+      st = sock_handler_write_str(sh, d_cmd, 1);
+      g_free(d_cmd);
+      if (st != 0)
          return 1;
-      }
 
       /* Send HTTP header */
-      //g_print("  sending HTTP header...\n");
       if (sock_handler_write(sh, Header, strlen(Header), 1) != 0) {
          return 1;
       }
 
-      //g_print("  sending page body...\n");
       st = send_bm_page(sh);
       if (st != 0) {
          char *err =
+            DOCTYPE
             "<HTML><body> Error on the bookmarks server...</body></html>";
          if (sock_handler_write(sh, err, strlen(err), 1) != 0) {
             return 1;
          }
       }
-      //g_print("  page body sent.\n");
       return 2;
    }
 
@@ -1783,7 +1768,6 @@ int main (void) {
             break;
       }
 
-      //g_print("Closing SockHandler\n");
       sock_handler_close(sh);
       sock_handler_free(sh);
 
