@@ -18,8 +18,11 @@
 #include <pthread.h>
 
 #include <ctype.h>           /* for tolower */
-#include <unistd.h>
+#include <errno.h>           /* for errno */
+#include <stdio.h>
 #include <stdlib.h>
+#include <string.h>
+#include <unistd.h>
 #include <sys/socket.h>
 #include <sys/stat.h>
 #include <sys/types.h>
@@ -27,13 +30,11 @@
 #include <sys/un.h>
 #include <dirent.h>
 #include <fcntl.h>
-#include <string.h>
 #include <time.h>
-#include <stdio.h>
 #include <signal.h>
-#include <errno.h>           /* for errno */
 #include <glib.h>
 
+#include "../dpip/dpip.h"
 #include "dpiutil.h"
 
 #define MAXNAMESIZE 30
@@ -80,7 +81,6 @@ static gint File_get_file(ClientInfo *Client,
 static gint File_get_dir(ClientInfo *Client,
                          const gchar *DirName,
                          const char *orig_url);
-static gchar *File_html_escape(const gchar *str);
 
 /*
  * Global variables
@@ -108,7 +108,7 @@ static void File_close(int fd)
  * only checks the few mime types that Dillo supports.
  *
  * 'Data' is a pointer to the first bytes of the raw data.
- * (this is a copy of a_Misc_get_content_type_from_data())
+ * (this is based on a_Misc_get_content_type_from_data())
  */
 static const gchar *File_get_content_type_from_data(void *Data, size_t Size)
 {
@@ -119,7 +119,9 @@ static const gchar *File_get_content_type_from_data(void *Data, size_t Size)
    };
    gint Type = 0;
    gchar *p = Data;
-   size_t i;
+   size_t i, non_ascci;
+
+   _MSG("File_get_content_type_from_data:: Size = %d\n", Size);
 
    /* HTML try */
    for (i = 0; i < Size && isspace(p[i]); ++i);
@@ -144,81 +146,23 @@ static const gchar *File_get_content_type_from_data(void *Data, size_t Size)
 
    /* Text */
    } else {
-      /* We'll assume ASCII if chars are below 128 (after all, this
-       * is a last resort when the server doesn't send Content-Type) */
-      Size = MIN (Size, 256);
-      for (i = 0; i < Size; i++)
-         if ((guchar) p[i] > 127)
-            break;
-      Type = (i < 12 || i < Size) ? 0 : 2;                      
+      /* We'll assume "text/plain" if the set of chars above 127 is <= 10
+       * in a 256-bytes sample.  Better heuristics are welcomed! :-) */
+      non_ascci = 0;
+      Size = MIN (Size, 256);    
+      for (i = 0; i < Size; i++) 
+         if ((unsigned char) p[i] > 127)
+            ++non_ascci;
+      if (Size == 256) {
+         Type = (non_ascci > 10) ? 0 : 2;
+      } else {
+         Type = (non_ascci > 0) ? 0 : 2;
+      }
    }
 
    return (Types[Type]);
 }
 
-/*
- * Escape characters as %XX sequences.
- * Return value: New string, or NULL if there's no need to escape.
- * (this is a copy of a_Misc_escape_chars())
- */
-static gchar *File_uri_escape(const gchar *str, gchar *esc_set)
-{
-   static const char *hex = "0123456789ABCDEF";
-   gchar *p = NULL;
-   GString *gstr;
-   gint i;
-
-   for (i = 0; str[i]; ++i)
-      if (str[i] <= 0x1F || str[i] == 0x7F || strchr(esc_set, str[i]))
-         break;
-
-   if (str[i]) {
-      /* needs escaping */
-      gstr = g_string_sized_new(64);
-      for (i = 0; str[i]; ++i) {
-         if (str[i] <= 0x1F || str[i] == 0x7F || strchr(esc_set, str[i])) {
-            g_string_append_c(gstr, '%');
-            g_string_append_c(gstr, hex[(str[i] >> 4) & 15]);
-            g_string_append_c(gstr, hex[str[i] & 15]);
-         } else {
-            g_string_append_c(gstr, str[i]);
-         }
-      }
-      p = gstr->str;
-      g_string_free(gstr, FALSE);
-   }
-   return p;
-}
-
-/*
- * Escape unsafe characters as html entities.
- * Return value: NULL if there's no need to escape, New string otherwise.
- */
-static gchar *File_html_escape(const gchar *str)
-{
-   static const char *unsafe_chars = "&<>\"'";
-   static const char *unsafe_rep[] =
-     { "&amp;", "&lt;", "&gt;", "&quot;", "&#39;" };
-
-   gchar *p;
-   GString *gstr;
-   gint i;
-
-   if ((p = strpbrk(str, unsafe_chars))) {
-      gstr = g_string_sized_new(64);
-
-      for (i = 0; str[i]; ++i) {
-         if ((p = strchr(unsafe_chars, str[i])))
-            g_string_append(gstr, unsafe_rep[p - unsafe_chars]);
-         else
-            g_string_append_c(gstr, str[i]);
-      }
-      p = gstr->str;
-      g_string_free(gstr, FALSE);
-   }
-   return p;
-}
-
 /*
  * Compare two FileInfo pointers
  * This function is used for sorting directories
@@ -328,21 +272,21 @@ static void File_dillodir_free(DilloDir *Ddir)
 static void File_print_parent_dir(ClientInfo *Client, const char *dirname)
 {
    if (strcmp(dirname, "/") != 0) {        /* Not the root dir */
-      char *parent = g_strdup(dirname);
-      char *p, *HCparent, *Cparent, *s1, *s2;
+      char *p, *parent, *HUparent, *Uparent;
 
-      /* cut trailing '/' */
+      parent = g_strdup(dirname);
+      /* cut trailing slash */
       parent[strlen(parent) - 1] = '\0';
       /* make 'parent' have the parent dir path */
       if ( (p = strrchr(parent, '/')) )
          *(p + 1) = '\0';
 
-      Cparent = (s1 = File_uri_escape(parent, "%#:' ")) ? s1: parent;
-      HCparent = (s2 = File_html_escape(Cparent)) ? s2 : Cparent;
+      Uparent = Escape_uri_str(parent, NULL);
+      HUparent = Escape_html_str(Uparent);
       sock_handler_printf(Client->sh, 0,
-         "<a href='file:%s'>Parent directory</a>", HCparent);
-      g_free(s2);
-      g_free(s1);
+         "<a href='file:%s'>Parent directory</a>", HUparent);
+      g_free(HUparent);
+      g_free(Uparent);
       g_free(parent);
    }
 }
@@ -365,7 +309,7 @@ static void File_print_mtime(ClientInfo *Client, time_t mtime)
    } else {
       sock_handler_printf(Client->sh, 0,
          "<td>%.3s&nbsp;%.2s&nbsp;%.5s", ds + 4, ds + 8,
-         // (more than 6 months old) ? year : hour;
+         /* (more than 6 months old) ? year : hour; */
          (time(NULL) - mtime > 15811200) ? ds + 20 : ds + 11);
    }
 }
@@ -379,9 +323,8 @@ static void File_info2html(ClientInfo *Client,
    gint size;
    char *sizeunits;
    char namebuf[MAXNAMESIZE + 1];
-   const char *ref, *Cref, *HCref, *Hname;
-   const gchar *cont, *filecont, *name = finfo->filename;
-   gchar *s1, *s2, *s3;
+   char *Uref, *HUref, *Hname;
+   const char *ref, *filecont, *name = finfo->filename;
 
    if (finfo->size <= 9999) {
       size = finfo->size;
@@ -396,13 +339,11 @@ static void File_info2html(ClientInfo *Client,
 
    /* we could note if it's a symlink... */
    if S_ISDIR (finfo->mode) {
-      cont = "application/directory";
       filecont = "Directory";
    } else if (finfo->mode & (S_IXUSR | S_IXGRP | S_IXOTH)) {
-      cont = "application/executable";
       filecont = "Executable";
    } else {
-      filecont = cont = File_content_type(finfo->full_path);
+      filecont = File_content_type(finfo->full_path);
       if (!filecont || !strcmp(filecont, "application/octet-stream"))
          filecont = "unknown";
    }
@@ -416,9 +357,9 @@ static void File_info2html(ClientInfo *Client,
    }
 
    /* escape problematic filenames */
-   Cref = (s1 = File_uri_escape(ref, "%#:' ")) ? s1 : ref;
-   HCref = (s2 = File_html_escape(Cref)) ? s2 : Cref;
-   Hname = (s3 = File_html_escape(name)) ? s3 : name;
+   Uref = Escape_uri_str(ref, NULL);
+   HUref = Escape_html_str(Uref);
+   Hname = Escape_html_str(name);
 
    if (Client->old_style) {
       char *dots = ".. .. .. .. .. .. .. .. .. .. .. .. .. .. .. .. ..";
@@ -427,7 +368,7 @@ static void File_info2html(ClientInfo *Client,
          "%s<a href='%s'>%s</a>"
          " %s"
          " %-11s%4d %-5s",
-         S_ISDIR (finfo->mode) ? ">" : " ", HCref, Hname,
+         S_ISDIR (finfo->mode) ? ">" : " ", HUref, Hname,
          dots + 50 - (ndots > 0 ? ndots : 0),
          filecont, size, sizeunits);
 
@@ -436,15 +377,15 @@ static void File_info2html(ClientInfo *Client,
          "<tr align=center %s><td>%s<td align=left><a href='%s'>%s</a>"
          "<td>%s<td>%d&nbsp;%s",
          (n & 1) ? "bgcolor=#dcdcdc" : "",
-         S_ISDIR (finfo->mode) ? ">" : " ", HCref, Hname,
+         S_ISDIR (finfo->mode) ? ">" : " ", HUref, Hname,
          filecont, size, sizeunits);
    }
    File_print_mtime(Client, finfo->mtime);
    sock_handler_printf(Client->sh, 0, "\n");
 
-   g_free(s3);
-   g_free(s2);
-   g_free(s1);
+   g_free(Hname);
+   g_free(HUref);
+   g_free(Uref);
 }
 
 /*
@@ -455,27 +396,28 @@ static void File_transfer_dir(ClientInfo *Client,
 {
    gint n;
    GList *list;
-   char *s1, *s2, *s3, *Hdirname, *Cdirname, *HCdirname;
+   char *d_cmd, *Hdirname, *Udirname, *HUdirname;
 
    /* Send DPI header */
-   sock_handler_printf(Client->sh, 1,
-      "<dpi cmd='start_send_page' url='%s'>\n", orig_url);
+   d_cmd = a_Dpip_build_cmd("cmd=%s url=%s", "start_send_page", orig_url);
+   sock_handler_write_str(Client->sh, d_cmd, 1);
+   g_free(d_cmd);
 
    /* Send page title */
-   Cdirname =
-      (s1 = File_uri_escape(Ddir->dirname, "%#:' ")) ? s1 : Ddir->dirname;
-   HCdirname = (s2 = File_html_escape(Cdirname)) ? s2 : Cdirname;
-   Hdirname = (s3 = File_html_escape(Ddir->dirname)) ? s3 : Ddir->dirname;
+   Udirname = Escape_uri_str(Ddir->dirname, NULL);
+   HUdirname = Escape_html_str(Udirname);
+   Hdirname = Escape_html_str(Ddir->dirname);
 
    sock_handler_printf(Client->sh, 0,
       "Content-Type: text/html\n\n"
+      "<!DOCTYPE HTML PUBLIC '-//W3C//DTD HTML 4.01 Transitional//EN'>\n"
       "<HTML>\n<HEAD>\n <BASE href='file:%s'>\n"
       " <TITLE>file:%s</TITLE>\n</HEAD>\n"
       "<BODY><H1>Directory listing of %s</H1>\n",
-      HCdirname, Hdirname, Hdirname);
-   g_free(s3);
-   g_free(s2);
-   g_free(s1);
+      HUdirname, Hdirname, Hdirname);
+   g_free(Hdirname);
+   g_free(HUdirname);
+   g_free(Udirname);
 
    if (Client->old_style) {
       sock_handler_printf(Client->sh, 0, "<pre>\n");
@@ -558,14 +500,19 @@ static const char *File_ext(const char *filename)
 static const char *File_content_type(const char *filename)
 {
    gint fd;
+   struct stat sb;
    const gchar *ct;
+   gchar buf[256];
+   ssize_t buf_size;
 
    if (!(ct = File_ext(filename))) {
       /* everything failed, let's analyze the data... */
       if ((fd = open(filename, O_RDONLY | O_NONBLOCK)) != -1) {
-         gchar buf[256];
-         ssize_t buf_size;
-         if ((buf_size = read(fd, buf, 256)) > 12 ) {
+         if ((buf_size = read(fd, buf, 256)) == 256 ) {
+            ct = File_get_content_type_from_data(buf, (size_t)buf_size);
+
+         } else if (stat(filename, &sb) != -1 &&
+                    buf_size > 0 && buf_size == sb.st_size) {
             ct = File_get_content_type_from_data(buf, (size_t)buf_size);
          }
          File_close(fd);
@@ -578,11 +525,12 @@ static const char *File_content_type(const char *filename)
 /*
  * Try to stat the file and determine if it's readable.
  */
-static void File_get(ClientInfo *Client, const char *filename, 
+static void File_get(ClientInfo *Client, const char *filename,
                      const char *orig_url)
 {
    int res;
    struct stat sb;
+   char *msg = NULL, *d_cmd;
 
    if (stat(filename, &sb) != 0) {
       /* stat failed, prepare a file-not-found error. */
@@ -596,14 +544,17 @@ static void File_get(ClientInfo *Client, const char *filename,
    }
 
    if (res == FILE_NOT_FOUND) {
-      sock_handler_printf(Client->sh, 1,
-         "<dpi cmd='send_status_message' msg='%s Not Found: %s'>",
-         S_ISDIR(sb.st_mode) ? "Directory" : "File", filename);
+      msg = g_strdup_printf("%s Not Found: %s",
+               S_ISDIR(sb.st_mode) ? "Directory" : "File", filename);
    } else if (res == FILE_NO_ACCESS) {
-      sock_handler_printf(Client->sh, 1,
-         "<dpi cmd='send_status_message' msg='"
-         "Failed to open the %s %s'>",
-         S_ISDIR(sb.st_mode) ? "directory" : "file", filename);
+      msg = g_strdup_printf("Access denied to %s: %s",
+               S_ISDIR(sb.st_mode) ? "Directory" : "File", filename);
+   }
+   if (msg) {
+      d_cmd = a_Dpip_build_cmd("cmd=%s msg=%s", "send_status_message", msg);
+      sock_handler_write_str(Client->sh, d_cmd, 1);
+      g_free(d_cmd);
+      g_free(msg);
    }
 }
 
@@ -643,25 +594,23 @@ static gint File_get_file(ClientInfo *Client,
 #define LBUF 16*1024
 
    const gchar *ct;
-   char buf[LBUF];
+   char buf[LBUF], *d_cmd;
    gint fd, st;
 
    if ( (fd = open(filename, O_RDONLY | O_NONBLOCK)) < 0)
       return FILE_NO_ACCESS;
 
-   /* Content type info: as we may misdetect a lot of files,
-    * every unknown type is rendered as "text/plain".
-    * todo: a better approach could be to detect&reject those types we know
-    * for sure we don't handle (as gzip, bzip, ELF, etc)
+   /* Content-Type info is based on filename extension. If there's no
+    * known extension, then we do data sniffing. If this doesn't lead
+    * to a conclusion, "application/octet-stream" is sent.
     */
-   ct = File_content_type(filename);
-   if (!ct || !strcmp(ct, "application/octet-stream"))
-      ct = "text/plain";
-
+   if (!(ct = File_content_type(filename)))
+      ct = "application/octet-stream";
 
    /* Send DPI command */
-   sock_handler_printf(Client->sh, 1,
-      "<dpi cmd='start_send_page' url='%s'>\n", orig_url);
+   d_cmd = a_Dpip_build_cmd("cmd=%s url=%s", "start_send_page", orig_url);
+   sock_handler_write_str(Client->sh, d_cmd, 1);
+   g_free(d_cmd);
 
    /* Send HTTP stream */
    sock_handler_printf(Client->sh, 0,
@@ -766,7 +715,7 @@ static char *File_normalize_path(const char *orig)
       if (str[1] == '\0' || str[1] == '/') {
          /* User wants the CWD */
          basename = g_get_current_dir();
-         str++; 
+         str++;
       } else if (str[1] == '.') {
          /* One level down from the CWD */
          char *tmp1 = g_get_current_dir();
@@ -802,8 +751,12 @@ static char *File_normalize_path(const char *orig)
  */
 static void File_toggle_html_style(ClientInfo *Client)
 {
+   char *d_cmd;
+
    OLD_STYLE = !OLD_STYLE;
-   sock_handler_printf(Client->sh, 0, "<dpi cmd='reload_request'>");
+   d_cmd = a_Dpip_build_cmd("cmd=%s", "reload_request");
+   sock_handler_write_str(Client->sh, d_cmd, 1);
+   g_free(d_cmd);
 }
 
 /*
@@ -892,15 +845,15 @@ static void *File_serve_client(void *data)
 
    /* Read the dpi command */
    dpip_tag = sock_handler_read(Client->sh);
-   MSG("dpip_tag = '%s'\n", dpip_tag);
+   MSG("dpip_tag={%s}\n", dpip_tag);
 
    if (dpip_tag) {
-      cmd = Get_attr_value(dpip_tag, strlen(dpip_tag), "cmd");
+      cmd = a_Dpip_get_attr(dpip_tag, strlen(dpip_tag), "cmd");
       if (cmd) {
          if (strcmp(cmd, "DpiBye") == 0) {
             DPIBYE = 1;
          } else {
-            url = Get_attr_value(dpip_tag, strlen(dpip_tag), "url");
+            url = a_Dpip_get_attr(dpip_tag, strlen(dpip_tag), "url");
             if (!url)
                MSG("file.dpi:: Failed to parse 'url'\n");
          }
@@ -988,11 +941,11 @@ int main(void)
 
    /* Arrange the cleanup function for abnormal terminations */
    if (signal (SIGINT, termination_handler) == SIG_IGN)
-     signal (SIGINT, SIG_IGN);  
+     signal (SIGINT, SIG_IGN);
    if (signal (SIGHUP, termination_handler) == SIG_IGN)
-     signal (SIGHUP, SIG_IGN);  
+     signal (SIGHUP, SIG_IGN);
    if (signal (SIGTERM, termination_handler) == SIG_IGN)
-     signal (SIGTERM, SIG_IGN); 
+     signal (SIGTERM, SIG_IGN);
 
    MSG("(v.1) accepting connections...\n");
 
@@ -1019,7 +972,7 @@ int main(void)
          accept(STDIN_FILENO, (struct sockaddr *)&spun, &address_size);
 
       if (temp_sock_descriptor == -1) {
-         perror("[accept]");    
+         perror("[accept]");
          break;
       }
 
