@@ -21,6 +21,13 @@
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
+ * As a special exception permission is granted to link the code of
+ * the https dillo plugin with the OpenSSL project's "OpenSSL"
+ * library, and distribute the linked executables, without including
+ * the source code for OpenSSL in the source distribution. You must
+ * obey the GNU General Public License, version 2, in all respects
+ * for all of the code used other than "OpenSSL".
+ *
  */
 
 /*
@@ -51,8 +58,10 @@
 #include <sys/time.h>
 #include <sys/stat.h>
 #include <glib.h>
+#include "../dpip/dpip.h"
 #include "dpiutil.h"
 
+#define ENABLE_SSL
 #undef ENABLE_SSL
 #ifdef ENABLE_SSL
 
@@ -89,7 +98,7 @@ static gint dialog_get_answer_number(void)
 
    /* Read the dpi command from STDIN */
    dpip_tag = sock_handler_read(sh);
-   response = Get_attr_value(dpip_tag, strlen(dpip_tag), "msg");
+   response = a_Dpip_get_attr(dpip_tag, strlen(dpip_tag), "msg");
    response_number = (response) ? strtol (response, NULL, 10) : -1;
    g_free(dpip_tag);
    g_free(response);
@@ -125,7 +134,7 @@ static void yes_ssl_support(void)
       /*Insufficient entropy.  Deal with it?*/
       g_printerr("Insufficient random entropy\n");
    }
-   
+
    /*Create context and SSL object*/
    if (exit_error == 0){
       ssl_context = SSL_CTX_new(SSLv23_client_method());
@@ -146,7 +155,7 @@ static void yes_ssl_support(void)
    }
 
    if (exit_error == 0){
-      snprintf(buf, 4095, "%s/.dillo/certs/", g_get_home_dir() );
+      g_snprintf(buf, 4095, "%s/.dillo/certs/", g_get_home_dir() );
       if (SSL_CTX_load_verify_locations(ssl_context, NULL, buf )==0){
          g_printerr("Error opening user x509 certificate location\n");
          exit_error = 1;
@@ -162,7 +171,7 @@ static void yes_ssl_support(void)
    }
 
    if (exit_error == 0){
-      /* Need to do the following if we want to deal with all 
+      /* Need to do the following if we want to deal with all
        * possible ciphers
        */
       SSL_set_cipher_list(ssl_connection, "ALL");
@@ -174,9 +183,9 @@ static void yes_ssl_support(void)
 
       /*Get the network address and command to be used*/
       dpip_tag = sock_handler_read(sh);
-      cmd = Get_attr_value(dpip_tag, strlen(dpip_tag), "cmd");
-      url = Get_attr_value(dpip_tag, strlen(dpip_tag), "url");
-      http_query = Get_attr_value(dpip_tag, strlen(dpip_tag), "query");   
+      cmd = a_Dpip_get_attr(dpip_tag, strlen(dpip_tag), "cmd");
+      url = a_Dpip_get_attr(dpip_tag, strlen(dpip_tag), "url");
+      http_query = a_Dpip_get_attr(dpip_tag, strlen(dpip_tag), "query");
 
       if (cmd == NULL || url == NULL || http_query == NULL){
          g_printerr("***Value of cmd, url or http_query is NULL"
@@ -218,14 +227,18 @@ static void yes_ssl_support(void)
       }
    }
 
-   if (exit_error == 0){
+   if (exit_error == 0) {
+      char *d_cmd;
+
       /*Send query we want*/
       SSL_write(ssl_connection, http_query, (int)strlen(http_query));
 
       /*Analyse response from server*/
 
       /*Send dpi command*/
-      sock_handler_printf(sh, 1, "<dpi cmd='start_send_page' url='%s'>", url);
+      d_cmd = a_Dpip_build_cmd("cmd=%s url=%s", "start_send_page", url);
+      sock_handler_write_str(sh, d_cmd, 1);
+      g_free(d_cmd);
 
       /*Send remaining data*/
 
@@ -255,7 +268,7 @@ static void yes_ssl_support(void)
 }
 
 /*
- * The following function attempts to open up a connection to the 
+ * The following function attempts to open up a connection to the
  * remote server and return the file descriptor number of the
  * socket.  Returns -1 in the event of an error
  */
@@ -271,8 +284,8 @@ static int get_network_connection(gchar * url)
    gchar * url_look_up = NULL;
 
    /*Determine how much of url we chop off as unneeded*/
-   if (strncasecmp(url, "https://", 8) == 0){
-      url_offset=8;
+   if (g_strncasecmp(url, "https://", 8) == 0){
+      url_offset = 8;
    }
 
    /*Find end of URL*/
@@ -289,7 +302,7 @@ static int get_network_connection(gchar * url)
    } else {
       url_look_up = url + url_offset;
    }
-  
+
    root_url = g_strdup(url_look_up);
    hp=gethostbyname(url_look_up);
 
@@ -318,7 +331,7 @@ static int get_network_connection(gchar * url)
 }
 
 
-/* This function is run only when the certificate cannot 
+/* This function is run only when the certificate cannot
  * be completely trusted.  This will notify the user and
  * allow the user to decide what to do.  It may save the
  * certificate to the user's .dillo directory if it is
@@ -330,24 +343,27 @@ static int handle_certificate_problem(SSL * ssl_connection)
    gint response_number;
    int retval = -1;
    long st;
-   char buf[4096];
+   char *cn, *cn_end;
+   char buf[4096], *d_cmd, *msg;
 
    X509 * remote_cert;
 
    remote_cert = SSL_get_peer_certificate(ssl_connection);
    if (remote_cert == NULL){
       /*Inform user that remote system cannot be trusted*/
-      sock_handler_printf(sh, 1,
-         "<dpi cmd='dialog' msg='%s' alt1='%s' alt2='%s'>",
+      d_cmd = a_Dpip_build_cmd(
+         "cmd=%s msg=%s alt1=%s alt2=%s",
+         "dialog",
          "The remote system is NOT presenting a certificate.\n"
          "This site CAN NOT be trusted. Sending data is NOT SAFE.\n"
          "What do I do?",
-         "Continue", "Cancel"
-      );
+         "Continue", "Cancel");
+      sock_handler_write_str(sh, d_cmd, 1);
+      g_free(d_cmd);
 
       /*Read the user's response*/
       response_number = dialog_get_answer_number();
-      
+
       /*Abort on anything but "Continue"*/
       if (response_number == 1){
          retval = 0;
@@ -363,17 +379,27 @@ static int handle_certificate_problem(SSL * ssl_connection)
       case X509_V_ERR_DEPTH_ZERO_SELF_SIGNED_CERT:
          /*Either self signed and untrusted*/
          /*Extract CN from certificate name information*/
-         strncpy(buf, (strstr(remote_cert->name, "/CN=") + 4),
-             (size_t)(strstr(strstr(remote_cert->name, "/CN=") + 4, "/") -
-                      strstr(remote_cert->name, "/CN=")- 4));
+         cn = strstr(remote_cert->name, "/CN=") + 4;
+         if (cn == NULL)
+            break;
+
+         if ((cn_end = strstr(cn, "/")) == NULL )
+            cn_end = cn + strlen(cn);
+
+         strncpy(buf, cn, (size_t) (cn_end - cn));
+
          /*Add terminating NULL*/
-         buf[strstr(strstr(remote_cert->name, "/CN=") + 4, "/") -
-             strstr(remote_cert->name, "/CN=")- 3] = 0;
-         sock_handler_printf(sh, 1,
-            "<dpi cmd='dialog' msg='%s%s' alt1='%s' alt2='%s' alt3='%s'>",
-            "The remote certificate is self-signed and untrusted.\n"
-            "For address:",
-            buf, "Continue", "Cancel", "Trust Certificate");
+         buf[cn_end - cn] = 0;
+
+         msg = g_strdup_printf("The remote certificate is self-signed and "
+                               "untrusted.\nFor address: %s", buf);
+         d_cmd = a_Dpip_build_cmd(
+            "cmd=%s msg=%s alt1=%s alt2=%s alt3=%s",
+            "dialog", msg, "Continue", "Cancel", "Trust Certificate");
+         sock_handler_write_str(sh, d_cmd, 1);
+         g_free(d_cmd);
+         g_free(msg);
+
          response_number = dialog_get_answer_number();
          switch (response_number){
             case 1:
@@ -394,12 +420,15 @@ static int handle_certificate_problem(SSL * ssl_connection)
          break;
       case X509_V_ERR_UNABLE_TO_GET_ISSUER_CERT:
       case X509_V_ERR_UNABLE_TO_DECODE_ISSUER_PUBLIC_KEY:
-         printf ("<dpi cmd='dialog' msg='%s' alt1='%s' alt2='%s'>",
-           "The issuer for the remote certificate cannot be found\n"
-           "The authenticity of the remote certificate cannot be trusted",
-           "Continue", "Cancel"
-         );
-         fflush(stdout);
+         d_cmd = a_Dpip_build_cmd(
+            "cmd=%s msg=%s alt1=%s alt2=%s",
+            "dialog",
+            "The issuer for the remote certificate cannot be found\n"
+            "The authenticity of the remote certificate cannot be trusted",
+            "Continue", "Cancel");
+         sock_handler_write_str(sh, d_cmd, 1);
+         g_free(d_cmd);
+
          response_number = dialog_get_answer_number();
          if (response_number == 1) {
             retval = 0;
@@ -410,12 +439,15 @@ static int handle_certificate_problem(SSL * ssl_connection)
       case X509_V_ERR_UNABLE_TO_DECRYPT_CRL_SIGNATURE:
       case X509_V_ERR_CERT_SIGNATURE_FAILURE:
       case X509_V_ERR_CRL_SIGNATURE_FAILURE:
-         printf ("<dpi cmd='dialog' msg='%s' alt1='%s' alt2='%s'>",
+         d_cmd = a_Dpip_build_cmd(
+            "cmd=%s msg=%s alt1=%s alt2=%s",
+            "dialog",
             "The remote certificate signature could not be read\n"
             "or is invalid and should not be trusted",
-            "Continue", "Cancel"
-         );
-         fflush (stdout);
+            "Continue", "Cancel");
+         sock_handler_write_str(sh, d_cmd, 1);
+         g_free(d_cmd);
+
          response_number = dialog_get_answer_number();
          if (response_number == 1) {
             retval = 0;
@@ -423,15 +455,18 @@ static int handle_certificate_problem(SSL * ssl_connection)
          break;
       case X509_V_ERR_CERT_NOT_YET_VALID:
       case X509_V_ERR_CRL_NOT_YET_VALID:
-         printf ("<dpi cmd='dialog' msg='%s' alt1='%s' alt2='%s'>",
+         d_cmd = a_Dpip_build_cmd(
+            "cmd=%s msg=%s alt1=%s alt2=%s",
+            "dialog",
             "Part of the remote certificate is not yet valid\n"
             "Certificates usually have a range of dates over which\n"
             "they are to be considered valid, and the certificate\n"
             "presented has a starting validity after today's date\n"
             "You should be cautious about using this site",
-            "Continue", "Cancel"
-         );
-         fflush (stdout);
+            "Continue", "Cancel");
+         sock_handler_write_str(sh, d_cmd, 1);
+         g_free(d_cmd);
+
          response_number = dialog_get_answer_number();
          if (response_number == 1) {
             retval = 0;
@@ -439,13 +474,15 @@ static int handle_certificate_problem(SSL * ssl_connection)
          break;
       case X509_V_ERR_CERT_HAS_EXPIRED:
       case X509_V_ERR_CRL_HAS_EXPIRED:
-         printf ("<dpi cmd='dialog' msg='%s' alt1='%s' alt2='%s'>",
+         d_cmd = a_Dpip_build_cmd(
+            "cmd=%s msg=%s alt1=%s alt2=%s",
+            "dialog",
             "The remote certificate has expired.  The certificate\n"
             "wasn't designed to last this long. You should avoid \n"
             "this site.",
-            "Continue", "Cancel"
-         );
-         fflush (stdout);
+            "Continue", "Cancel");
+         sock_handler_write_str(sh, d_cmd, 1);
+         g_free(d_cmd);
          response_number = dialog_get_answer_number();
          if (response_number == 1) {
             retval = 0;
@@ -455,14 +492,16 @@ static int handle_certificate_problem(SSL * ssl_connection)
       case X509_V_ERR_ERROR_IN_CERT_NOT_AFTER_FIELD:
       case X509_V_ERR_ERROR_IN_CRL_LAST_UPDATE_FIELD:
       case X509_V_ERR_ERROR_IN_CRL_NEXT_UPDATE_FIELD:
-         printf ("<dpi cmd='dialog' msg='%s' alt1='%s' alt2='%s'>",
+         d_cmd = a_Dpip_build_cmd(
+            "cmd=%s msg=%s alt1=%s alt2=%s",
+            "dialog",
             "There was an error in the certificate presented.\n"
             "Some of the certificate data was improperly formatted\n"
             "making it impossible to determine if the certificate\n"
             "is valid.  You should not trust this certificate.",
-            "Continue", "Cancel"
-         );
-         fflush (stdout);
+            "Continue", "Cancel");
+         sock_handler_write_str(sh, d_cmd, 1);
+         g_free(d_cmd);
          response_number = dialog_get_answer_number();
          if (response_number == 1) {
             retval = 0;
@@ -473,14 +512,16 @@ static int handle_certificate_problem(SSL * ssl_connection)
       case X509_V_ERR_CERT_UNTRUSTED:
       case X509_V_ERR_CERT_REJECTED:
       case X509_V_ERR_KEYUSAGE_NO_CERTSIGN:
-         printf ("<dpi cmd='dialog' msg='%s' alt1='%s' alt2='%s'>",
+         d_cmd = a_Dpip_build_cmd(
+            "cmd=%s msg=%s alt1=%s alt2=%s",
+            "dialog",
             "One of the certificates in the chain is being used\n"
             "incorrectly (possibly due to configuration problems\n"
             "with the remote system.  The connection should not\n"
             "be trusted",
-            "Continue", "Cancel"
-         );
-         fflush(stdout);
+            "Continue", "Cancel");
+         sock_handler_write_str(sh, d_cmd, 1);
+         g_free(d_cmd);
          response_number = dialog_get_answer_number();
          if (response_number == 1) {
             retval = 0;
@@ -489,18 +530,24 @@ static int handle_certificate_problem(SSL * ssl_connection)
       case X509_V_ERR_SUBJECT_ISSUER_MISMATCH:
       case X509_V_ERR_AKID_SKID_MISMATCH:
       case X509_V_ERR_AKID_ISSUER_SERIAL_MISMATCH:
-         printf ("<dpi cmd='dialog' msg='%s' alt1='%s' alt2='%s'>",
+         d_cmd = a_Dpip_build_cmd(
+            "cmd=%s msg=%s alt1=%s alt2=%s",
+            "dialog",
             "Some of the information presented by the remote system\n"
             "does not match other information presented\n"
             "This may be an attempt to evesdrop on communications",
-            "Continue", "Cancel"
-         );
+            "Continue", "Cancel");
+         sock_handler_write_str(sh, d_cmd, 1);
+         g_free(d_cmd);
       default:             /*Need to add more options later*/
-         sock_handler_printf(sh, 1,
-            "<dpi cmd='dialog' msg='%s%ld' alt1='%s' alt2='%s'>",
-            "The remote certificate cannot be verified",
-            st, "Continue", "Cancel"
-         );
+         msg = g_strdup_printf(
+            "The remote certificate cannot be verified (code %ld)", st);
+         d_cmd = a_Dpip_build_cmd(
+            "cmd=%s msg=%s alt1=%s alt2=%s",
+            "dialog", msg, "Continue", "Cancel");
+         sock_handler_write_str(sh, d_cmd, 1);
+         g_free(d_cmd);
+         g_free(msg);
          response_number = dialog_get_answer_number();
          /*abort on anything but "Continue"*/
          if (response_number == 1){
@@ -527,13 +574,13 @@ static int save_certificate_home(X509 * cert)
    int retval = 1;
 
    /*Attempt to create .dillo/certs blindly - check later*/
-   snprintf(buf,4096,"%s/.dillo/", g_get_home_dir());
+   g_snprintf(buf,4096,"%s/.dillo/", g_get_home_dir());
    mkdir(buf, 01777);
-   snprintf(buf,4096,"%s/.dillo/certs/", g_get_home_dir());
+   g_snprintf(buf,4096,"%s/.dillo/certs/", g_get_home_dir());
    mkdir(buf, 01777);
 
    do{
-      snprintf(buf,4096,"%s/.dillo/certs/%lx.%u",
+      g_snprintf(buf,4096,"%s/.dillo/certs/%lx.%u",
          g_get_home_dir(), X509_subject_name_hash(cert), i);
 
       fp=fopen(buf, "r");
@@ -572,6 +619,7 @@ static int save_certificate_home(X509 * cert)
 static void no_ssl_support(void)
 {
    gchar *dpip_tag = NULL, *cmd = NULL, *url = NULL, *http_query = NULL;
+   char *d_cmd;
 
    /* Read the dpi command from STDIN */
    dpip_tag = sock_handler_read(sh);
@@ -579,9 +627,9 @@ static void no_ssl_support(void)
    g_printerr("{In https.filter.dpi}\n");
    g_printerr("no_ssl_support version\n");
 
-   cmd = Get_attr_value(dpip_tag, strlen(dpip_tag), "cmd");
-   url = Get_attr_value(dpip_tag, strlen(dpip_tag), "url");
-   http_query = Get_attr_value(dpip_tag, strlen(dpip_tag), "query");
+   cmd = a_Dpip_get_attr(dpip_tag, strlen(dpip_tag), "cmd");
+   url = a_Dpip_get_attr(dpip_tag, strlen(dpip_tag), "url");
+   http_query = a_Dpip_get_attr(dpip_tag, strlen(dpip_tag), "query");
 
    g_printerr("{ cmd: %s}\n", cmd);
    g_printerr("{ url: %s}\n", url);
@@ -589,7 +637,9 @@ static void no_ssl_support(void)
 
    g_printerr("{ sending dpip cmd...}\n");
 
-   sock_handler_printf(sh, 1, "<dpi cmd='start_send_page' url='%s'>", url);
+   d_cmd = a_Dpip_build_cmd("cmd=%s url=%s", "start_send_page", url);
+   sock_handler_write_str(sh, d_cmd, 1);
+   g_free(d_cmd);
 
    g_printerr("{ dpip cmd sent.}\n");
 
@@ -597,6 +647,7 @@ static void no_ssl_support(void)
 
    sock_handler_printf(sh, 1,
       "Content-type: text/html\n\n"
+      "<!DOCTYPE HTML PUBLIC '-//W3C//DTD HTML 4.01 Transitional//EN'>\n"
       "<html><body><pre>\n"
       "<b>Hi!\n\n"
       "  This is the https dpi that just got a request to send\n"
@@ -606,7 +657,7 @@ static void no_ssl_support(void)
       "  <b>*** Dillo's prototype plugin for https support"
       " is disabled now ***</b>\n\n"
       "  If you want to test this <b>alpha</b> support code, just remove\n"
-      "  line 56 from https.c, recompile and reinstall.\n\n"
+      "  line 65 from https.c, recompile and reinstall.\n\n"
       "  (beware that this https support is very limited now)\n\n"
       "  To use https and SSL, you must have \n"
       "  the OpenSSL development libraries installed.  Check your\n"
@@ -633,7 +684,7 @@ static void no_ssl_support(void)
 /*---------------------------------------------------------------------------*/
 int main(void)
 {
-   /* Initialize the SockHandler for this filter dpi */        
+   /* Initialize the SockHandler for this filter dpi */
    sh = sock_handler_new(STDIN_FILENO, STDOUT_FILENO, 8*1024);
 
 #ifdef ENABLE_SSL
