@@ -30,6 +30,7 @@
 #include <glib.h>
 #include "dpid_common.h"
 #include "dpid.h"
+#include "../dpip/dpip.h"
 
 #ifdef TEST
 #include "testdat.h"
@@ -130,14 +131,14 @@ char *get_dpi_dir(char *dpidrc)
 static int send_service_list(int sock, struct dp *dpi_attr_list, int srv_num)
 {
    int i;
-   char *start_tag = "<dpi cmd='send_data' msg='", *end_tag = "'>";
    gchar *buf;
    ssize_t wlen = 0;
 
    for (i = 0; i < srv_num && wlen != -1; i++) {
-      buf = g_strconcat(start_tag, dpi_attr_list[i].id, end_tag, NULL);
-      wlen = write(sock, buf, strlen(buf));
-      g_free(buf);
+      d_cmd = a_Dpip_build_cmd("cmd=%s msg=%s",
+                               "send_data", dpi_attr_list[i].id);
+      wlen = write(sock, d_cmd, strlen(d_cmd));
+      g_free(d_cmd);
    }
    if (wlen == -1) {
       ERRMSG("send_service_list", "write", errno);
