@@ -31,6 +31,8 @@
 #include "dpi_service.h"
 #include "misc_new.h"
 
+#include "../dpip/dpip.h"
+
 #define QUEUE 5
 
 volatile sig_atomic_t caught_sigchld = 0;
@@ -555,8 +557,7 @@ void dpi_sigchld(int sig)
 /*! Called by main loop when caught_sigchld == 1 */
 void handle_sigchld(void)
 {
-   // pid_t pid;
-   int i, status; //, num_active;
+   int i, status;
 
    /* For all of the dpis in the current list
     *    add the ones that have exited to the set of sockets being
@@ -595,12 +596,15 @@ void est_dpi_sigchld(void)
  */
 void stop_active_dpis(struct dp *dpi_attr_list, int numdpis)
 {
+   static char *DpiBye_cmd = NULL;
    int i, dpi_socket;
    struct sockaddr_un dpi_addr;
-   char *DpiBye_cmd = "<dpi cmd='DpiBye'>";
    struct sockaddr_un sa;
    size_t sun_path_len, addr_len;
 
+   if (!DpiBye_cmd)
+      DpiBye_cmd = a_Dpip_build_cmd("cmd=%s", "DpiBye");
+
    sun_path_len = sizeof(sa.sun_path);
    addr_len = sizeof(dpi_addr);
 
@@ -645,7 +649,7 @@ void ignore_dpi_sockets(struct dp *dpi_attr_list, int numdpis)
 
 /*! Registers available dpis and stops active non-filter dpis.
  * Called when dpid receives
- * <dpi cmd='register' service='all'>
+ * cmd='register' service='all'
  * command
  * \Return
  * Number of available dpis
@@ -671,12 +675,15 @@ int register_all_cmd(char *sockdir)
  */
 char *get_message(int sock, char *dpi_tag)
 {
-   char *msg;
+   char *msg, *d_cmd;
 
-   msg = a_Misc_get_attr_value(dpi_tag, strlen(dpi_tag), "msg");
+   msg = a_Dpip_get_attr(dpi_tag, strlen(dpi_tag), "msg");
    if (msg == NULL) {
       ERRMSG("get_message", "failed to parse msg\n", 0);
-      (void) CKD_WRITE(sock, tag_prserr);
+      d_cmd = a_Dpip_build_cmd("cmd=%s msg=%s",
+                               "DpiError", "Failed to parse request");
+      (void) CKD_WRITE(sock, d_cmd);
+      g_free(d_cmd);
    }
    return (msg);
 }
@@ -684,11 +691,11 @@ char *get_message(int sock, char *dpi_tag)
 /*!
  * Send socket path that matches dpi_id to client
  */
-void send_sockpath(gint sock, gchar * dpi_tag, struct dp *dpi_attr_list)
+void send_sockpath(gint sock, gchar *dpi_tag, struct dp *dpi_attr_list)
 {
    gint i;
    gchar *dpi_id;
-   char *req_rply = NULL, *dpi_cmd = "<dpi cmd='send_data' msg='%s'>";
+   char *d_cmd;
 
    g_return_if_fail((dpi_id = get_message(sock, dpi_tag)) != NULL);
 
@@ -700,13 +707,17 @@ void send_sockpath(gint sock, gchar * dpi_tag, struct dp *dpi_attr_list)
       if (access(dpi_attr_list[i].path, F_OK) == -1) {
          ERRMSG("send_sockpath", "access", errno);
          fprintf(stderr, " - %s\n", dpi_attr_list[i].sockpath);
-         (void) CKD_WRITE(sock, tag_dpi_unavailerr);
+         d_cmd = a_Dpip_build_cmd("cmd=%s msg=%s",
+                                  "DpiError", "Plugin currently unavailable");
+         (void) CKD_WRITE(sock, d_cmd);
+         g_free(d_cmd);
       } else {
-         req_rply = g_strdup_printf(dpi_cmd, dpi_attr_list[i].sockpath);
-         (void) CKD_WRITE(sock, req_rply);
+         d_cmd = a_Dpip_build_cmd("cmd=%s msg=%s",
+                                  "send_data", dpi_attr_list[i].sockpath);
+         (void) CKD_WRITE(sock, d_cmd);
+         g_free(d_cmd);
       }
    }
 
    g_free(dpi_id);
-   g_free(req_rply);
 }
