@@ -27,15 +27,17 @@
 #include "dpi.h"
 #include "dpi_socket_dir.h"
 #include "misc_new.h"
+#include "../dpip/dpip.h"
 
 sigset_t mask_sigchld;
 
+
 /* Start a dpi filter plugin after accepting the pending connection
  * \Return
  * \li Child process ID on success
  * \li 0 on failure
  */
-int start_filter_plugin(struct dp dpi_attr)
+static int start_filter_plugin(struct dp dpi_attr)
 {
    int newsock, old_stdout=-1, old_stdin=-1;
    socklen_t csz;
@@ -87,7 +89,7 @@ int start_filter_plugin(struct dp dpi_attr)
    return pid;
 }
 
-void start_server_plugin(struct dp dpi_attr)
+static void start_server_plugin(struct dp dpi_attr)
 {
    if (dup2(dpi_attr.socket, STDIN_FILENO) == -1) {
       ERRMSG("start_plugin", "dup2", errno);
@@ -111,7 +113,7 @@ void start_server_plugin(struct dp dpi_attr)
  * \Return
  * pointer to dynamically allocated request tag
  */
-char *get_request(int sock)
+static char *get_request(int sock)
 {
    char *req, buf[10];
    size_t buflen;
@@ -144,9 +146,9 @@ char *get_request(int sock)
  * \Return
  * command code on success, -1 on failure
  */
-int get_command(int sock, char *dpi_tag, struct dp *dpi_attr_list)
+static int get_command(int sock, char *dpi_tag, struct dp *dpi_attr_list)
 {
-   char *cmd;
+   char *cmd, *d_cmd;
    int COMMAND;
 
    if (dpi_tag == NULL) {
@@ -154,12 +156,15 @@ int get_command(int sock, char *dpi_tag, struct dp *dpi_attr_list)
       return (-1);
    }
 
-   cmd = a_Misc_get_attr_value(dpi_tag, (int) strlen(dpi_tag), "cmd");
+   cmd = a_Dpip_get_attr(dpi_tag, strlen(dpi_tag), "cmd");
 
    if (cmd == NULL) {
-      ERRMSG("get_command", "a_Misc_get_attr_value", 0);
+      ERRMSG("get_command", "a_Dpip_get_attr", 0);
       fprintf(stderr, ": dpid failed to parse cmd in %s\n", dpi_tag);
-      (void) CKD_WRITE(sock, tag_prserr);
+      d_cmd = a_Dpip_build_cmd("cmd=%s msg=%s",
+                               "DpiError", "Failed to parse request");
+      (void) CKD_WRITE(sock, d_cmd);
+      g_free(d_cmd);
       COMMAND = -1;
    } else if (strcmp("DpiBye", cmd) == 0) {
       COMMAND = BYE_CMD;
@@ -177,10 +182,27 @@ int get_command(int sock, char *dpi_tag, struct dp *dpi_attr_list)
    return (COMMAND);
 }
 
+/*
+ * Check whether a dpi server is running
+ */
+int server_is_running(char *server_id)
+{
+   int i;
+
+   /* Search in the set of running servers */
+   for (i = 0; i < numdpis; i++) {
+      if (!dpi_attr_list[i].filter && dpi_attr_list[i].pid > 1 &&
+          strcmp(dpi_attr_list[i].id, server_id) == 0)
+         return 1;
+   }
+   return 0;
+}
+
+
 /*
  * Get MAX open FD limit (yes, it's tricky --Jcid).
  */
-int get_open_max()
+static int get_open_max(void)
 {
 #ifdef OPEN_MAX
    return OPEN_MAX;
@@ -196,7 +218,7 @@ int get_open_max()
  * \li Add a dpid_idle_timeout variable to dpidrc
  * \bug Infinite loop if plugin crashes before it accepts a connection
  */
-int main()
+int main(void)
 {
    int i, n = 0, open_max;
    char *dirname = NULL, *sockdir = NULL;
@@ -206,13 +228,14 @@ int main()
    fd_set selected_set;
 
    dpi_attr_list = NULL;
-   //daemon(0,0); /* Use 0,1 for feedback */
+   /* daemon(0,0); */ /* Use 0,1 for feedback */
    /* todo: call setsid() ?? */
 
-   /* avoid problems with umount and with file creation */
-   umask(0);
+   /* Allow read and write access, but only for the user.
+    * todo: can this cause trouble with umount? */
+   umask(0077);
    /* todo: make dpid work on any directory. */
-   // chdir("/");
+   /* chdir("/"); */
 
    /* close inherited file descriptors */
    open_max = get_open_max();
@@ -220,7 +243,7 @@ int main()
       a_Misc_close_fd(i);
 
    /* this sleep used to unmask a race condition */
-   // sleep(2);
+   /* sleep(2); */
 
    dpi_errno = no_errors;
 
@@ -251,7 +274,8 @@ int main()
       }
    }
    numsocks = init_all_dpi_sockets(dpi_attr_list, sockdir);
-   //est_terminator(); /* Do we still want to clean up on an abnormal exit? */
+   /* est_terminator();  Do we still want to clean up on an abnormal exit? */
+
    est_dpi_sigchld();
 
    (void) sigemptyset(&mask_sigchld);
@@ -273,7 +297,14 @@ int main()
          select_timeout.tv_usec = 0;
          selected_set = sock_set;
          n = select(FD_SETSIZE, &selected_set, NULL, NULL, &select_timeout);
-         if ( n == 0 ) { /* select timed out, exit */
+         if ( n == 0 ) { /* select timed out, try to exit */
+            /* BUG: This is a workaround for dpid not to exit when the
+             * downloads server is active. The proper way to handle it is with
+             * a dpip command that asks the server whether it's busy.
+             * Note: the cookies server may lose session info too. */
+            if (server_is_running("downloads"))
+               continue;
+
             stop_active_dpis(dpi_attr_list, numdpis);
             cleanup(sockdir);
             exit(0);
@@ -318,10 +349,15 @@ int main()
                register_all_cmd(sockdir);
                break;
             case UNKNOWN_CMD:
-               (void) CKD_WRITE(sock, tag_cmderr);
+               {
+               char *d_cmd = a_Dpip_build_cmd("cmd=%s msg=%s",
+                                              "DpiError", "Unknown command");
+               (void) CKD_WRITE(sock, d_cmd);
+               g_free(d_cmd);
                ERRMSG("main", "Unknown command", 0);
                fprintf(stderr, " for request: %s\n", req);
                break;
+               }
             case -1:
                ERRMSG("main", "get_command failed\n", 0);
                break;
