@@ -71,35 +71,6 @@ gchar *a_Misc_prepend_user_home(const char *file)
    return (g_strconcat(a_Misc_get_home(), "/", file, NULL));
 }
 
-/*!
- * This was copied from dpi/bm_srv12.c
- * - Task: given a tag and an attribute name, return its value.
- *       (character stuffing is removed here)
- * - Return value: the attribute value, or NULL if not present or malformed.
- */
-char *a_Misc_get_attr_value(char *tag, size_t tagsize, char *attrname)
-{
-   char *p, *q, *ltag, quote, *start, *val = NULL;
-
-   ltag = g_strndup(tag, tagsize);
-   if ((p = strstr(ltag, attrname)) &&
-       (p = strchr(p, '=')) && (p = strpbrk(p, "'\""))) {
-      quote = *p;
-      start = ++p;
-      while ((q = strchr(p, quote)) && q[1] == quote)
-         p = q + 2;
-      if (q) {
-         val = g_strndup(start, (guint)(q - start));
-         for (p = q = val; (*q = *p); ++p, ++q)
-            if ((*p == '"' || *p == '\'') && p[1] == p[0])
-               ++p;
-      }
-   }
-   g_free(ltag);
-
-   return val;
-}
-
 /*
  * Read a line of text up to the newline character, store it into a newly
  * allocated string and return it.
@@ -178,7 +149,7 @@ char *a_Misc_readtag(int sock)
    tag = NULL;
    buf[0] = '\0';
    buflen = sizeof(buf) / sizeof(buf[0]);
-   // new start
+   /* new start */
    tag = (char *) g_malloc(tagmem + 1);
    for (i = 0; (rdln = read(sock, &c, 1)) != 0; i++) {
       if (i == tagmem) {
@@ -192,7 +163,7 @@ char *a_Misc_readtag(int sock)
          break;
       }
    }
-   // new end
+   /* new end */
    if (rdln == -1) {
       ERRMSG("a_Misc_readtag", "read", errno);
    }
