@@ -57,8 +57,7 @@ typedef struct {
 IOData_t* a_IO_new(gint op, gint fd);
 void a_IO_set_buf(IOData_t *io, void *Buf, size_t BufSize);
 void a_IO_add_buf(IOData_t *io, void *Buf, size_t BufSize);
-void a_IO_ccc(int Op, int Branch, int Dir, ChainLink *Info,
-              void *Data1, void *Data2);
+/* Note: a_IO_ccc() is defined in Url.h together with the *_ccc() set */
 
 void a_IO_write_chunk(gint FD, void *Key, void *Buf, size_t BufSize);
 
