@@ -149,14 +149,6 @@ static char *Splash=
 "    <td>&nbsp;&nbsp;\n"
 "    <td>\n"
 "     <a href='http://www.theregister.co.uk/index.html'>The Register</a>\n"
-"    <tr>\n"
-"    <td>&nbsp;&nbsp;\n"
-"    <td>\n"
-"     <a href='http://www.infoanarchy.org/'>InfoAnarchy</a>\n"
-"    <tr>\n"
-"    <td>&nbsp;&nbsp;\n"
-"    <td>\n"
-"     <a href='http://www.linuxforkids.org/'>Linux4Kids</a>\n"
 "    </table>\n"
 "    </table>\n"
 "  </table>\n"
@@ -263,21 +255,23 @@ static char *Splash=
 "<tr>\n"
 " <td bgcolor='#CCCCCC'>\n"
 "  <h4>Release overview</h4>\n"
-"  15 June 2005\n"
+"  April 26, 2006\n"
 "<tr>\n"
 " <td bgcolor='#FFFFFF'>\n"
 "  <table border='0' cellspacing='0' cellpadding='5'>\n"
 "  <tr>\n"
 "   <td>\n"
 "<p>\n"
-"The  new dillo-" VERSION " is a bugfix release that addresses a couple\n"
-"of security issues. Users are advised to upgrade ASAP.\n"
+"This release is hopefully the last from the GTK1 series. The port to\n"
+"<a href='http://www.fltk.org/'>FLTK2</a> is almost finished now!\n"
 "<p>\n"
-"This release is one of the last on the GTK1-based dillo series.\n"
-"Currently the main development is focused on a "
-"<a href='http://www.fltk.org/'>FLTK2</a>-based dillo prototype.\n"
-"If everything goes as planned, this new branch will become our next release!"
-" (stay tuned).\n"
+"This release comes with extensive work on the plugins side. A new dpip\n"
+"library, a new FLTK2-based GUI for downloads!, a \"<code>data:</code>\"\n"
+"URI handler, a important bug fix in the FTP plugin, and a cookies server\n"
+"dpi that enables all Dillo instances to use cookies at the same time!\n"
+"<p>\n"
+"There's also the \"<code>./configure --disable-threaded-dns</code>\"\n"
+"option (mainly for some non reentrant BSDs), among many other improvements.\n"
 "<p>\n"
 "Remember that dillo project uses a release model where every new\n"
 "browser shall be better than the former.\n"
@@ -302,11 +296,29 @@ static char *Splash=
 "  <tr>\n"
 "   <td>\n"
 "<ul>\n"
-"<li>Added a \"small\" dillorc option for panel size.\n"
-"<li>Set \"file:\" to work as URI for current directory.\n"
-"<li>Fixed the shell escaping code in the ftp plugin.\n"
-"<li>Added some checks for sane values in html.c.\n"
-"<li>Added URL filtering to the ftp and downloads dpis to avoid SMTP hacks.\n"
+"<li> Designed and implemented a dpi protocol library (libDpip.a in /dpip).\n"
+"<li> Ported the bookmarks, download, file, https, ftp and hello plugins,\n"
+"     plus the dpid daemon and the rest of the source tree to use it.\n"
+"<li> Improved the dpi buffer reception to handle split buffers (This was\n"
+"     required for handling arbitrary data streams with dpip).\n"
+"<li> Fixed a serious bug with the FTP plugin that led to two downloads\n"
+"     of the same file when left-clicking a non-viewable file.\n"
+"<li> Improved the accuracy of the illegal-character error reporting\n"
+"     for URLs.\n"
+"<li> Added dpi/downloads.cc (Default FLTK2-based GUI for downloads dpi).\n"
+"<li> Added \"./configure --disable-dlgui\" to build without FLTK2-GUI\n"
+"     downloads.\n"
+"<li> Fixed dpip's tag syntax and its parsing to accept any value string.\n"
+"<li> Added DOCTYPE parsing (for better bug-meter error messages).\n"
+"<li> Fixed bookmarks dpi to escape ' in URLs and &<>\"' in titles\n"
+"     (BUG#655).\n"
+"<li> Added a check for malicious image sizes in IMG tags.\n"
+"\n"
+"<li> Added a datauri dpi to handle \"data:\" URIs (RFC-2397).\n"
+"<li> Moved the cookies management into a dpi server: cookies.dpi.\n"
+"<li> Removed the restriction of only one dillo with cookies enabled!\n"
+"<li> Added \"./configure --disable-threaded-dns\" (for some non\n"
+"     reentrant BSDs).\n"
 "</ul>\n"
 "  </table>\n"
 "</table>\n"
