@@ -44,7 +44,8 @@
 
 typedef struct {
    const DilloUrl *Url;      /* Cached Url. Url is used as a primary Key */
-   const char *Type;         /* MIME type string */
+   char *TypeDet;            /* MIME type string (detected from data) */
+   char *TypeHdr;            /* MIME type string as from the HTTP Header */
    GString *Header;          /* HTTP header */
    const DilloUrl *Location; /* New URI for redirects */
    void *Data;               /* Pointer to raw data */
@@ -203,7 +204,8 @@ static void Cache_client_dequeue(CacheClient_t *Client, gint Key)
 static void Cache_entry_init(CacheData_t *NewEntry, const DilloUrl *Url)
 {
    NewEntry->Url = a_Url_dup(Url);
-   NewEntry->Type = NULL;
+   NewEntry->TypeDet = NULL;
+   NewEntry->TypeHdr = NULL;
    NewEntry->Header = g_string_new("");
    NewEntry->Location = NULL;
    NewEntry->Data = NULL;
@@ -246,7 +248,8 @@ static CacheData_t *Cache_entry_add(const DilloUrl *Url)
 static void Cache_entry_free(CacheData_t *entry)
 {
    a_Url_free((DilloUrl *)entry->Url);
-   g_free((gchar *)entry->Type);
+   g_free(entry->TypeDet);
+   g_free(entry->TypeHdr);
    g_string_free(entry->Header, TRUE);
    a_Url_free((DilloUrl *)entry->Location);
    g_free(entry->Data);
@@ -264,13 +267,14 @@ static gint Cache_entry_remove_raw(CacheData_t *entry, DilloUrl *url)
 
    /* There MUST NOT be any clients */
    g_return_val_if_fail(
-      !g_slist_find_custom(ClientQueue, url, Cache_client_url_cmp), 0);
+      !g_slist_find_custom(
+         ClientQueue, (gpointer)entry->Url, Cache_client_url_cmp), 0);
 
    /* remove from DelayedQueue */
    DelayedQueue = g_slist_remove(DelayedQueue, entry);
 
    /* remove from dicache */
-   a_Dicache_invalidate_entry(url);
+   a_Dicache_invalidate_entry(entry->Url);
 
    /* remove from cache */
    g_hash_table_remove(CacheHash, entry->Url);
@@ -300,7 +304,7 @@ static void Cache_entry_remove(CacheData_t *entry, DilloUrl *url)
       a_Cache_ccc(OpAbort, 2, BCK, InfoAnswer, NULL, NULL);
    } else {
       DEBUG_MSG(2, "## Aborting raw2\n");
-      Cache_entry_remove_raw(entry, url);
+      Cache_entry_remove_raw(entry, NULL);
    }
 }
 
@@ -502,6 +506,7 @@ static char *Cache_parse_field(const char *header, const char *fieldname)
    return NULL;
 }
 
+#ifndef DISABLE_COOKIES
 /*
  * Extract multiple fields from the header.
  */
@@ -535,6 +540,7 @@ static GList *Cache_parse_multiple_fields(const char *header,
    }
    return fields;
 }
+#endif /* !DISABLE_COOKIES */
 
 /*
  * Scan, allocate, and set things according to header info.
@@ -555,10 +561,10 @@ static void Cache_parse_header(CacheData_t *entry, IOData_t *io, gint HdrLen)
       /* 30x: URL redirection */
       entry->Flags |= CA_Redirect;
       if ( header[11] == '1' )
-         entry->Flags |= CA_ForceRedirect;  // 301 Moved Permanently
+         entry->Flags |= CA_ForceRedirect;  /* 301 Moved Permanently */
       else if ( header[11] == '2' )
-         entry->Flags |= CA_TempRedirect;   // 302 Temporal Redirect
-     
+         entry->Flags |= CA_TempRedirect;   /* 302 Temporal Redirect */
+
       location_str = Cache_parse_field(header, "Location");
       entry->Location = a_Url_new(location_str, URL_STR_(entry->Url), 0, 0, 0);
       g_free(location_str);
@@ -607,8 +613,10 @@ static void Cache_parse_header(CacheData_t *entry, IOData_t *io, gint HdrLen)
    if ( (Type = Cache_parse_field(header, "Content-Type")) == NULL ) {
      MSG_HTTP("Server didn't send Content-Type in header.\n");
    } else {
-      entry->Type = Type;
-      entry->Flags |= CA_GotContentType;
+      entry->TypeHdr = Type;
+      /* This Content-Type is not trusted. It's checked against real data
+       * in Cache_process_queue(); only then CA_GotContentType becomes true.
+       */
    }
 }
 
@@ -765,6 +773,19 @@ static gint Cache_redirect(CacheData_t *entry, gint Flags, BrowserWindow *bw)
    return 0;
 }
 
+/*
+ * Check whether a URL scheme is downloadable.
+ * Return: 1 enabled, 0 disabled.
+ */
+int Cache_download_enabled(const DilloUrl *url)
+{
+   if (!strcasecmp(URL_SCHEME(url), "http") ||
+       !strcasecmp(URL_SCHEME(url), "https") ||
+       !strcasecmp(URL_SCHEME(url), "ftp") )
+      return 1;
+   return 0;
+}
+
 /*
  * Don't process data any further, but let the cache fill the entry.
  * (Currently used to handle WEB_RootUrl redirects,
@@ -807,6 +828,8 @@ static void Cache_process_queue(CacheData_t *entry)
    BrowserWindow *Client_bw = NULL;
    static gboolean Busy = FALSE;
    gboolean AbortEntry = FALSE;
+   gboolean OfferDownload = FALSE;
+   gboolean TypeMismatch = FALSE;
 
    if ( Busy )
       DEBUG_MSG(5, "FATAL!:*** >>>> Cache_process_queue Caught busy!!!\n");
@@ -816,7 +839,12 @@ static void Cache_process_queue(CacheData_t *entry)
       st = a_Misc_get_content_type_from_data(
               entry->Data, entry->ValidSize, &Type);
       if (st == 0 || entry->Flags & CA_GotData) {
-         entry->Type = g_strdup(Type);
+         if (a_Misc_content_type_check(entry->TypeHdr, Type) < 0) {
+            MSG_HTTP("Content-Type '%s' doesn't match the real data.\n",
+                     entry->TypeHdr);
+            TypeMismatch = TRUE;
+         }
+         entry->TypeDet = g_strdup(Type);
          entry->Flags |= CA_GotContentType;
       } else
          return;  /* i.e., wait for more data */
@@ -834,6 +862,9 @@ static void Cache_process_queue(CacheData_t *entry)
                a_Interface_msg(Client_bw, "");
                entry->Flags |= CA_MsgErased;
             }
+            if (TypeMismatch)
+               a_Interface_msg(Client_bw,"HTTP warning: Content-Type '%s' "
+                               "doesn't match the real data.", entry->TypeHdr);
             if (entry->Flags & CA_Redirect) {
                if (!Client->Callback) {
                   Client->Callback = a_Cache_null_client;
@@ -850,21 +881,36 @@ static void Cache_process_queue(CacheData_t *entry)
 
          /* Set the client function */
          if (!Client->Callback) {
-            st = a_Web_dispatch_by_type(entry->Type, ClientWeb,
-                                        &Client->Callback, &Client->CbData);
-            if (st == -1) {
-               /* MIME type is not viewable */
+            if (TypeMismatch) {
+               AbortEntry = TRUE;
                Client->Callback = a_Cache_null_client;
-               Client->CbData = NULL;
-               if (ClientWeb->flags & WEB_RootUrl) {
-                  /* Unhandled MIME type, prepare a download offer... */
-                  AbortEntry = TRUE;
-                  a_Interface_remove_client(Client_bw, Client->Key);
-                  Cache_client_dequeue(Client, NULLKey);
-                  --i; /* Keep the index value in the next iteration */
-                  continue;
+            } else {
+               st = a_Web_dispatch_by_type(
+                       entry->TypeHdr ? entry->TypeHdr : entry->TypeDet,
+                       ClientWeb, &Client->Callback, &Client->CbData);
+               if (st == -1) {
+                  /* MIME type is not viewable */
+                  Client->Callback = a_Cache_null_client;
+                  if (ClientWeb->flags & WEB_RootUrl) {
+                     /* Unhandled MIME type, prepare a download offer... */
+                     AbortEntry = TRUE;
+                     OfferDownload = TRUE;
+                  } else {
+                     /* TODO: Resource Type not handled.
+                      * Not aborted to avoid multiple connections on the same
+                      * resource. A better idea is to abort the connection and
+                      * to keep a failed-resource flag in the cache entry. */
+                     MSG_HTTP("Unhandled MIME type: <%s>\n",
+                              entry->TypeHdr ? entry->TypeHdr:entry->TypeDet);
+                  }
                }
             }
+            if (AbortEntry) {
+               a_Interface_remove_client(Client_bw, Client->Key);
+               Cache_client_dequeue(Client, NULLKey);
+               --i; /* Keep the index value in the next iteration */
+               continue;
+            }
          }
 
          /* Send data to our client */
@@ -896,9 +942,10 @@ static void Cache_process_queue(CacheData_t *entry)
    } /* for */
 
    if (AbortEntry) {
-      /* Abort the entry, remove it from cache, and offer download.
+      /* Abort the entry, remove it from cache, and maybe offer download.
        * (the dialog is made before 'entry' is freed) */
-      a_Interface_offer_link_download(Client_bw, entry->Url);
+      if (OfferDownload && Cache_download_enabled(entry->Url))
+         a_Interface_offer_link_download(Client_bw, entry->Url);
       Cache_entry_remove(entry, NULL);
    }
 
@@ -1109,13 +1156,13 @@ void a_Cache_ccc(int Op, int Branch, int Dir, ChainLink *Info,
             IOData_t *io2;
 
             Info->LocalKey = Data2;
-            if ((entry = Cache_entry_search(Data2)))    // Is Data2 valid?
+            if ((entry = Cache_entry_search(Data2)))    /* Is Data2 valid? */
                entry->CCCAnswer = Info;
 
             /* Link IO to receive the answer */
             io2 = a_IO_new(IORead, *(int*)Data1);
             a_IO_set_buf(io2, NULL, IOBufLen);
-            io2->ExtData = Data2;       // We have it as LocalKey
+            io2->ExtData = Data2;       /* We have it as LocalKey */
             a_Chain_link_new(Info, a_Cache_ccc, BCK, a_IO_ccc, 2, 2);
             a_Chain_bcb(OpStart, Info, io2, NULL);
             break;
