@@ -15,17 +15,18 @@
 /*
  * Flag Defines
  */
-#define CA_GotHeader       (1)  /* True if header is completely got */
-#define CA_GotLength       (2)  /* True if we have all Data in cache */
-#define CA_GotData         (4)  /* True if we have all Data in cache */
-#define CA_FreeData        (8)  /* Free the cache Data on close */
-#define CA_Redirect       (16)  /* Data actually points to a redirect */
-#define CA_ForceRedirect  (32)  /* Unconditional redirect */
-#define CA_TempRedirect   (64)  /* Temporal redirect */
-#define CA_NotFound      (128)  /* True if remote server didn't find the URL */
-#define CA_Stopped       (256)  /* True if the entry has been stopped */
-#define CA_MsgErased     (512)  /* Used to erase the bw's status bar */
-#define CA_RedirectLoop (1024)  /* Redirect loop */
+#define CA_GotHeader       (1)  /* True after header is completely got */
+#define CA_GotContentType  (2)  /* True after Content-Type is known */
+#define CA_GotLength       (4)  /* True if Content-Length is known */
+#define CA_GotData         (8)  /* True if we have all Data in cache */
+#define CA_FreeData       (16)  /* Free the cache Data on close */
+#define CA_Redirect       (32)  /* Data actually points to a redirect */
+#define CA_ForceRedirect  (64)  /* Unconditional redirect */
+#define CA_TempRedirect  (128)  /* Temporal redirect */
+#define CA_NotFound      (256)  /* True if remote server didn't find the URL */
+#define CA_Stopped       (512)  /* True if the entry has been stopped */
+#define CA_MsgErased    (1024)  /* Used to erase the bw's status bar */
+#define CA_RedirectLoop (2048)  /* Redirect loop */
 
 /*
  * Callback type for cache clients
