@@ -236,7 +236,7 @@ void a_Dicache_invalidate_entry(const DilloUrl *Url)
 void a_Dicache_callback(int Op, CacheClient_t *Client)
 {
    /* todo: Handle Op = CA_Abort (to show what was got)  --Jcid */
-   gint i;
+   guint i;
    DilloWeb *Web = Client->Web;
    DilloImage *Image = Web->Image;
    DICacheEntry *DicEntry = a_Dicache_get_entry(Web->url);
@@ -251,11 +251,10 @@ void a_Dicache_callback(int Op, CacheClient_t *Client)
             DicEntry->version, DicEntry->width, DicEntry->height,
             DicEntry->type);
       }
-      if ( DicEntry->State == DIC_Write ) {
-
-         for ( i = 0; i < DicEntry->height; ++i )
-            if ( a_Bitvec_get_bit(DicEntry->BitVec, i) &&
-                 !a_Bitvec_get_bit(Image->BitVec, i) )
+      if (DicEntry->State == DIC_Write) {
+         for (i = 0; i < DicEntry->height; ++i)
+            if (a_Bitvec_get_bit(DicEntry->BitVec, (gint)i) &&
+                !a_Bitvec_get_bit(Image->BitVec, (gint)i) )
                a_Image_write(
                   Image, DicEntry->ImageBuffer + i*DicEntry->width*3, i,FALSE);
 
@@ -273,7 +272,7 @@ void a_Dicache_callback(int Op, CacheClient_t *Client)
  * (By now, we'll use the image information despite the html tags --Jcid)
  */
 void a_Dicache_set_parms(DilloUrl *url, gint version, DilloImage *Image,
-                         gint width, gint height, DilloImgType type)
+                         guint width, guint height, DilloImgType type)
 {
    DICacheEntry *DicEntry;
    size_t Size = width * height * 3;
@@ -291,7 +290,7 @@ void a_Dicache_set_parms(DilloUrl *url, gint version, DilloImage *Image,
    DicEntry->width = width;
    DicEntry->height = height;
    DicEntry->type = type;
-   DicEntry->BitVec = a_Bitvec_new(height);
+   DicEntry->BitVec = a_Bitvec_new((gint)height);
    DicEntry->State = DIC_SetParms;
 
    dicache_size_total += Size;
@@ -308,7 +307,7 @@ void a_Dicache_set_parms(DilloUrl *url, gint version, DilloImage *Image,
  * Implement the set_cmap method for the Image
  */
 void a_Dicache_set_cmap(DilloUrl *url, gint version, DilloImage *Image,
-                        const guchar *cmap, gint num_colors,
+                        const guchar *cmap, guint num_colors,
                         gint num_colors_max, gint bg_index)
 {
    DICacheEntry *DicEntry = Dicache_get_entry_version(url, version);
@@ -318,7 +317,7 @@ void a_Dicache_set_cmap(DilloUrl *url, gint version, DilloImage *Image,
    g_free(DicEntry->cmap);
    DicEntry->cmap = g_new0(guchar, 3 * num_colors_max);
    memcpy(DicEntry->cmap, cmap, 3 * num_colors);
-   if (bg_index >= 0 && bg_index < num_colors) {
+   if (bg_index >= 0 && (guint)bg_index < num_colors) {
       DicEntry->cmap[bg_index * 3]     = (Image->bg_color >> 16) & 0xff;
       DicEntry->cmap[bg_index * 3 + 1] = (Image->bg_color >> 8) & 0xff;
       DicEntry->cmap[bg_index * 3 + 2] = (Image->bg_color) & 0xff;
@@ -336,7 +335,7 @@ void a_Dicache_set_cmap(DilloUrl *url, gint version, DilloImage *Image,
  * x  : horizontal offset? (always zero)
  */
 void a_Dicache_write(DilloImage *Image, DilloUrl *url, gint version,
-                     const guchar *buf, gint x, gint Y)
+                     const guchar *buf, gint x, guint Y)
 {
    DICacheEntry *DicEntry;
 
@@ -347,7 +346,7 @@ void a_Dicache_write(DilloImage *Image, DilloUrl *url, gint version,
 
    a_Image_write(Image, buf, Y, TRUE);
    DicEntry->Y = Y;
-   a_Bitvec_set_bit(DicEntry->BitVec, Y);
+   a_Bitvec_set_bit(DicEntry->BitVec, (gint)Y);
    DicEntry->State = DIC_Write;
 }
 
