@@ -36,9 +36,9 @@
 
 
 /*
- * Note: comment the following line for debugging or gprof profiling.
+ * Uncomment the following line for debugging or gprof profiling.
  */
-#define G_DNS_THREADED
+/* #undef D_DNS_THREADED */
 
 /*
  * Uncomment the following line for libc5 optimization
@@ -47,10 +47,10 @@
 
 
 /* Maximum dns resolving threads */
-#ifdef G_DNS_THREADED
-#  define G_DNS_MAX_SERVERS 4
+#ifdef D_DNS_THREADED
+#  define D_DNS_MAX_SERVERS 4
 #else
-#  define G_DNS_MAX_SERVERS 1
+#  define D_DNS_MAX_SERVERS 1
 #endif
 
 
@@ -61,7 +61,7 @@ typedef struct {
    GSList *addr_list;    /* IP address */
    char *hostname;       /* Adress to resolve */
    guint timeout_id;     /* gtk timeout function ID */
-#ifdef G_DNS_THREADED
+#ifdef D_DNS_THREADED
    pthread_t th1;        /* Thread id */
 #endif
 } DnsServer;
@@ -86,7 +86,7 @@ static gboolean Dns_timeout_client(gpointer data);
 /*
  * Local Data
  */
-static DnsServer dns_server[G_DNS_MAX_SERVERS];
+static DnsServer dns_server[D_DNS_MAX_SERVERS];
 static gint num_servers;
 static GDnsCache *dns_cache;
 static gint dns_cache_size, dns_cache_size_max;
@@ -188,7 +188,12 @@ void a_Dns_init(void)
 {
    gint i;
 
-   DEBUG_MSG(5, "dillo_dns_init: Here we go!\n");
+#ifdef D_DNS_THREADED
+   DEBUG_MSG(5, "dillo_dns_init: Here we go! (threaded)\n");
+#else
+   DEBUG_MSG(5, "dillo_dns_init: Here we go! (not threaded)\n");
+#endif
+
    dns_queue_size = 0;
    dns_queue_size_max = 16;
    dns_queue = g_new(GDnsQueue, dns_queue_size_max);
@@ -197,7 +202,7 @@ void a_Dns_init(void)
    dns_cache_size_max = 16;
    dns_cache = g_new(GDnsCache, dns_cache_size_max);
 
-   num_servers = G_DNS_MAX_SERVERS;
+   num_servers = D_DNS_MAX_SERVERS;
 
    /* Initialize servers data */
    for (i = 0; i < num_servers; ++i) {
@@ -207,7 +212,7 @@ void a_Dns_init(void)
       dns_server[i].addr_list = NULL;
       dns_server[i].hostname = NULL;
       dns_server[i].timeout_id = 0;
-#ifdef G_DNS_THREADED
+#ifdef D_DNS_THREADED
       dns_server[i].th1 = (pthread_t) -1;
 #endif
    }
@@ -246,6 +251,7 @@ static GSList *Dns_note_hosts(GSList *list, int af, struct hostent *host)
    return list;
 }
 
+#ifdef D_DNS_THREADED
 /*
  *  Server function (runs on its own thread)
  */
@@ -289,9 +295,9 @@ static void *Dns_server(void *data)
 
    return NULL;                 /* (avoids a compiler warning) */
 }
+#endif
 
-
-#ifndef G_DNS_THREADED
+#ifndef D_DNS_THREADED
 /*
  *  Blocking server-function (it doesn't use threads)
  */
@@ -338,7 +344,7 @@ static void Dns_blocking_server(void)
  */
 static void Dns_server_req(gint channel, const char *hostname)
 {
-#ifdef G_DNS_THREADED
+#ifdef D_DNS_THREADED
    static pthread_attr_t thrATTR;
    static gint thrATTRInitialized = 0;
 #endif
@@ -354,7 +360,7 @@ static void Dns_server_req(gint channel, const char *hostname)
       gtk_timeout_add(200, (GtkFunction)Dns_timeout_client,
                       GINT_TO_POINTER(dns_server[channel].channel));
 
-#ifdef G_DNS_THREADED
+#ifdef D_DNS_THREADED
    /* set the thread attribute to the detached state */
    if (!thrATTRInitialized) {
       pthread_attr_init(&thrATTR);
