@@ -43,10 +43,10 @@ static gint styles_num = 0;
 
 /* Used by a_Dw_style_numtostr(). */
 static const char
-   *roman_I0[] ={"I","II","III","IV","V","VI","VII","VIII","IX"},
-   *roman_I1[] ={"X","XX","XXX","XL","L","LX","LXX","LXXX","XC"},
-   *roman_I2[] ={"C","CC","CCC","CD","D","DC","DCC","DCCC","CM"},
-   *roman_I3[] ={"M","MM","MMM","MMMM"};
+   *roman_I0[] ={"","I","II","III","IV","V","VI","VII","VIII","IX"},
+   *roman_I1[] ={"","X","XX","XXX","XL","L","LX","LXX","LXXX","XC"},
+   *roman_I2[] ={"","C","CC","CCC","CD","D","DC","DCC","DCCC","CM"},
+   *roman_I3[] ={"","M","MM","MMM","MMMM"};
 
 
 static gint Dw_style_font_equal (gconstpointer v1, gconstpointer v2);
@@ -993,43 +993,33 @@ void a_Dw_style_numtostr (gint num,
 {
    int i3, i2, i1, i0;
    gboolean low = FALSE;
+   int start_ch = 'A';
 
    switch(list_style_tyle){
    case DW_STYLE_LIST_STYLE_TYPE_LOWER_ALPHA:
-      low = TRUE;
+      start_ch = 'a';
    case DW_STYLE_LIST_STYLE_TYPE_UPPER_ALPHA:
       i0 = num - 1;
-      i1 = i0/26; i2 = i1/26;
-      i0 %= 26;   i1 %= 26;
-      if (i2 > 26) /* more than 26*26*26=17576 elements ? */
+      i1 = i0/26 - 1; i2 = i1/26 - 1;
+      if (i2 > 25) /* more than 26+26^2+26^3=18278 elements ? */
          sprintf(buf, "****.");
-      else if (i2)
-         sprintf(buf, "%c%c%c.", 'A'+i2-1,'A'+i1-1,'A'+i0);
-      else if (i1)
-         sprintf(buf, "%c%c.", 'A'+i1-1,'A'+i0);
       else
-         sprintf(buf, "%c.", 'A'+i0);
-      if (low)
-         g_strdown(buf);
+         sprintf(buf, "%c%c%c.",
+                 i2<0 ? ' ' : start_ch + i2%26,
+                 i1<0 ? ' ' : start_ch + i1%26,
+                 i0<0 ? ' ' : start_ch + i0%26);
       break;
    case DW_STYLE_LIST_STYLE_TYPE_LOWER_ROMAN:
       low = TRUE;
    case DW_STYLE_LIST_STYLE_TYPE_UPPER_ROMAN:
-      i0 = num - 1;
+      i0 = num;
       i1 = i0/10; i2 = i1/10; i3 = i2/10;
       i0 %= 10;   i1 %= 10;   i2 %= 10;
-      if (i3 > 4) /* more than 4999 elements ? */
+      if (num < 0 || i3 > 4) /* more than 4999 elements ? */
          sprintf(buf, "****.");
-      else if (i3)
-         sprintf(buf, "%s%s%s%s.", roman_I3[i3-1], roman_I2[i2-1],
-                 roman_I1[i1-1], roman_I0[i0]);
-      else if (i2)
-         sprintf(buf, "%s%s%s.", roman_I2[i2-1],
-                 roman_I1[i1-1], roman_I0[i0]);
-      else if (i1)
-         sprintf(buf, "%s%s.", roman_I1[i1-1], roman_I0[i0]);
       else
-         sprintf(buf, "%s.", roman_I0[i0]);
+         g_snprintf(buf, buflen, "%s%s%s%s.", roman_I3[i3], roman_I2[i2],
+                    roman_I1[i1], roman_I0[i0]);
       if (low)
          g_strdown(buf);
       break;
