@@ -73,14 +73,14 @@ struct _DwTable
 
    gint cur_row, cur_col;
 
-   gint32 *col_width;    // num_cols members
+   gint32 *col_width;   /*  num_cols members */
    gint num_cols;
    gint num_col_width_max;
 
-   gint32 *cum_height;  // row cumulative height array:
-   gint num_rows;       //  num_cum_height is (num_rows + 1),
-   gint cum_height_max; //  cum_height[0] is always SPACING,
-                        //  cum_height[num_rows] is the total table height
+   gint32 *cum_height;  /*  row cumulative height array: */
+   gint num_rows;       /*  num_cum_height is (num_rows + 1), */
+   gint cum_height_max; /*  cum_height[0] is always SPACING, */
+                        /*  cum_height[num_rows] is the total table height */
 
    DwStyle **row_style;
    gint row_style_max;
