@@ -28,7 +28,6 @@
 static void Dw_widget_init                (DwWidget *widget);
 static void Dw_widget_class_init          (DwWidgetClass *klass);
 
-static void Dw_widget_shutdown            (GtkObject *object);
 static void Dw_widget_destroy             (GtkObject *object);
 
 static void Dw_widget_real_size_request   (DwWidget *widget,
@@ -152,7 +151,6 @@ static void Dw_widget_class_init (DwWidgetClass *klass)
                      2, GTK_TYPE_POINTER, GTK_TYPE_GDK_EVENT);
    gtk_object_class_add_signals (object_class, widget_signals, LAST_SIGNAL);
 
-   object_class->shutdown = Dw_widget_shutdown;
    object_class->destroy = Dw_widget_destroy;
 
    klass->size_request = Dw_widget_real_size_request;
@@ -173,24 +171,6 @@ static void Dw_widget_class_init (DwWidgetClass *klass)
 }
 
 
-
-/*
- * Standard Gtk+ function
- */
-static void Dw_widget_shutdown (GtkObject *object)
-{
-   DwWidget *widget;
-
-   widget = DW_WIDGET (object);
-
-   if (widget->parent)
-      Dw_container_remove (DW_CONTAINER (widget->parent), widget);
-   else
-      Dw_gtk_viewport_remove_dw (GTK_DW_VIEWPORT (widget->viewport));
-
-   parent_class->shutdown (object);
-}
-
 /*
  * Standard Gtk+ function
  */
@@ -209,6 +189,11 @@ static void Dw_widget_destroy (GtkObject *object)
    if (widget->style)
       a_Dw_style_unref (widget->style);
 
+   if (widget->parent)
+      Dw_container_remove (DW_CONTAINER (widget->parent), widget);
+   else
+      Dw_gtk_viewport_remove_dw (GTK_DW_VIEWPORT (widget->viewport));
+
    parent_class->destroy (object);
 }
 
