@@ -50,6 +50,12 @@ struct _DilloHtmlLB {
 };
 
 
+typedef enum {
+   DT_NONE,
+   DT_HTML,
+   DT_XHTML
+} DilloHtmlDocumentType;
+
 typedef enum {
    DILLO_HTML_PARSE_MODE_INIT,
    DILLO_HTML_PARSE_MODE_STASH,
@@ -199,6 +205,9 @@ struct _DilloHtml {
    size_t CurrTagOfs;
    size_t OldTagOfs, OldTagLine;
 
+   DilloHtmlDocumentType DocType; /* as given by DOCTYPE tag */
+   float DocTypeVersion;          /* HTML or XHTML version number */
+
    DilloHtmlState *stack;
    gint stack_top;        /* Index to the top of the stack [0 based] */
    gint stack_max;
