@@ -46,6 +46,7 @@
 
 #include "pixmaps.h"
 #include <gdk/gdkkeysyms.h>
+#include "../dpip/dpip.h"
 
 #define DEBUG_LEVEL 0
 #include "debug.h"
@@ -382,7 +383,7 @@ static void Interface_cub_selection_received(GtkWidget *widget,
        DEBUG_MSG (1, "Selection retrieval failed\n");
        return;
    }
-   
+
    damn_string = g_strndup((gchar *)selection_data->data,
                            selection_data->length);
    gtk_entry_set_text(GTK_ENTRY(bw->location), damn_string);
@@ -1254,7 +1255,7 @@ void a_Interface_quit_all(void)
       gtk_widget_destroy(bws[i]->main_window);
 
    g_free(bws);
-   
+
    g_free(open_dialog_last_dirname);
    g_free(save_dialog_last_dirname);
 }
@@ -1694,7 +1695,7 @@ static void Interface_file_save_link(GtkWidget *widget, BrowserWindow *bw)
 {
    const gchar *name;
    const DilloUrl *url;
-   gchar *cmd, *buf, *s_url, *s_name;
+   gchar *cmd, *buf;
    FILE *out;
    gint buf_size;
 
@@ -1707,15 +1708,10 @@ static void Interface_file_save_link(GtkWidget *widget, BrowserWindow *bw)
 
    if (!a_Capi_get_buf(url, &buf, &buf_size)) {
       /* Not cached, ask the downloads server to get it */
-      s_url = a_Misc_stuff_chars(URL_STR(url), "'\"");
-      s_name = a_Misc_stuff_chars(name, "'\"");
-      cmd = g_strdup_printf(
-               "<dpi cmd='download' url='%s' destination='%s'>\n",
-               s_url, s_name);
+      cmd = a_Dpip_build_cmd("cmd=%s url=%s destination=%s",
+                             "download", URL_STR(url), name);
       a_Capi_dpi_send_cmd(NULL, bw, cmd, "downloads", 1);
       g_free(cmd);
-      g_free(s_name);
-      g_free(s_url);
 
    } else {
       /* Cached! Save from the cache */
@@ -1738,13 +1734,22 @@ static void Interface_file_save_link(GtkWidget *widget, BrowserWindow *bw)
  */
 static char *Interface_make_save_name(const DilloUrl *url)
 {
-   gchar *FileName, *FilenameWithDir;
+   int i;
+   gchar *FileName, *FilenameWithDir, *o, *n;
 
    if ((FileName = strrchr(URL_PATH(url), '/')))
       FileName = g_strndup(FileName + 1, MIN(strlen(FileName + 1), 80));
    else
       FileName = g_strdup("");
 
+   /* Replace %20 and ' ' with '_' in Filename */
+   o = n = FileName;
+   for (i = 0; o[i]; i++) {
+      *n++ = (o[i] == ' ' || (o[i] == '%' && o[i+1] == '2' && o[i+2] == '0')) ?
+             i+=2, '_' : o[i];
+   }
+   *n = 0;
+
    if (save_dialog_last_dirname) {
       if (*FileName) {
         gchar *dirpart = g_dirname(save_dialog_last_dirname);
