@@ -4,7 +4,6 @@
 
 gchar *a_Misc_prepend_user_home(const char *file);
 gchar *a_Misc_escape_chars(const gchar *str, gchar *esc_set);
-gchar *a_Misc_stuff_chars(const gchar *str, const gchar *esc_set);
 gchar *a_Misc_stristr(char *src, char *str);
 gchar *a_Misc_expand_tabs(const char *str);
 gchar *a_Misc_strsep(char **orig, const char *delim);
@@ -12,7 +11,8 @@ gchar *a_Misc_strsep(char **orig, const char *delim);
 gint *a_Misc_strsplitpos(const gchar *str, const gchar *delim);
 gint *a_Misc_strsplitposdup(gint *pos);
 gint a_Misc_get_content_type_from_data(void *Data, size_t Size,
-                                       const gchar **PT);
+                                       const char **PT);
+int a_Misc_content_type_check(const char *EntryType, const char *DetectedType);
 gint a_Misc_parse_geometry(gchar *geom, gint *x, gint *y, gint *w, gint *h);
 gchar *a_Misc_encode_base64(const gchar *in);
 
