@@ -46,42 +46,44 @@
  * (non '_'-ended macros MUST use these for initialization sake)
  */
 /* these MAY return NULL: */
-#define URL_SCHEME_(u)         u->scheme
-#define URL_AUTHORITY_(u)      u->authority
-#define URL_PATH_(u)           u->path
-#define URL_QUERY_(u)          u->query
-#define URL_FRAGMENT_(u)       u->fragment
-#define URL_HOST_(u)           a_Url_hostname(u)
-#define URL_DATA_(u)           u->data
-#define URL_ALT_(u)            u->alt
-#define URL_STR_(u)            a_Url_str(u)
+#define URL_SCHEME_(u)              u->scheme
+#define URL_AUTHORITY_(u)           u->authority
+#define URL_PATH_(u)                u->path
+#define URL_QUERY_(u)               u->query
+#define URL_FRAGMENT_(u)            u->fragment
+#define URL_HOST_(u)                a_Url_hostname(u)
+#define URL_DATA_(u)                u->data
+#define URL_ALT_(u)                 u->alt
+#define URL_STR_(u)                 a_Url_str(u)
 /* these return an integer */
-#define URL_PORT_(u)           (URL_HOST(u) ? u->port : u->port)
-#define URL_FLAGS_(u)          u->flags
-#define URL_POSX_(u)           u->scrolling_position_x
-#define URL_POSY_(u)           u->scrolling_position_y
-#define URL_ILLEGAL_CHARS_(u)  url->illegal_chars
+#define URL_PORT_(u)                (URL_HOST(u) ? u->port : u->port)
+#define URL_FLAGS_(u)               u->flags
+#define URL_POSX_(u)                u->scrolling_position_x
+#define URL_POSY_(u)                u->scrolling_position_y
+#define URL_ILLEGAL_CHARS_(u)       url->illegal_chars
+#define URL_ILLEGAL_CHARS_SPC_(u)   url->illegal_chars_spc
 
 /*
  * Access methods that always return a string:
  * When the "empty" and "undefined" concepts of RFC-2396 are irrelevant to
  * the caller, and a string is required, use these methods instead:
  */
-#define NPTR2STR(p)           ((p) ? (p) : "")
-#define URL_SCHEME(u)         NPTR2STR(URL_SCHEME_(u))
-#define URL_AUTHORITY(u)      NPTR2STR(URL_AUTHORITY_(u))
-#define URL_PATH(u)           NPTR2STR(URL_PATH_(u))
-#define URL_QUERY(u)          NPTR2STR(URL_QUERY_(u))
-#define URL_FRAGMENT(u)       NPTR2STR(URL_FRAGMENT_(u))
-#define URL_HOST(u)           NPTR2STR(URL_HOST_(u))
-#define URL_DATA(u)           NPTR2STR(URL_DATA_(u))
-#define URL_ALT(u)            NPTR2STR(URL_ALT_(u))
-#define URL_STR(u)            NPTR2STR(URL_STR_(u))
-#define URL_PORT(u)           URL_PORT_(u)
-#define URL_FLAGS(u)          URL_FLAGS_(u)
-#define URL_POSX(u)           URL_POSX_(u)
-#define URL_POSY(u)           URL_POSY_(u)
-#define URL_ILLEGAL_CHARS(u)  URL_ILLEGAL_CHARS_(u)
+#define NPTR2STR(p)                 ((p) ? (p) : "")
+#define URL_SCHEME(u)               NPTR2STR(URL_SCHEME_(u))
+#define URL_AUTHORITY(u)            NPTR2STR(URL_AUTHORITY_(u))
+#define URL_PATH(u)                 NPTR2STR(URL_PATH_(u))
+#define URL_QUERY(u)                NPTR2STR(URL_QUERY_(u))
+#define URL_FRAGMENT(u)             NPTR2STR(URL_FRAGMENT_(u))
+#define URL_HOST(u)                 NPTR2STR(URL_HOST_(u))
+#define URL_DATA(u)                 NPTR2STR(URL_DATA_(u))
+#define URL_ALT(u)                  NPTR2STR(URL_ALT_(u))
+#define URL_STR(u)                  NPTR2STR(URL_STR_(u))
+#define URL_PORT(u)                 URL_PORT_(u)
+#define URL_FLAGS(u)                URL_FLAGS_(u)
+#define URL_POSX(u)                 URL_POSX_(u)
+#define URL_POSY(u)                 URL_POSY_(u)
+#define URL_ILLEGAL_CHARS(u)        URL_ILLEGAL_CHARS_(u)
+#define URL_ILLEGAL_CHARS_SPC(u)    URL_ILLEGAL_CHARS_SPC_(u)
 
 
 
@@ -105,12 +107,12 @@ extern "C" {
 struct _DilloUrl {
    GString *url_string;
    const gchar *buffer;
-   const gchar *scheme;            //
-   const gchar *authority;         //
-   const gchar *path;              // These are references only
-   const gchar *query;             // (no need to free them)
-   const gchar *fragment;          //
-   const gchar *hostname;          //
+   const gchar *scheme;            /* */
+   const gchar *authority;         /* */
+   const gchar *path;              /* These are references only */
+   const gchar *query;             /* (no need to free them) */
+   const gchar *fragment;          /* */
+   const gchar *hostname;          /* */
    gint port;
    gint flags;
    const gchar *data;              /* POST */
@@ -119,6 +121,7 @@ struct _DilloUrl {
    gint32 scrolling_position_x, scrolling_position_y;
                                    /* remember position of visited urls */
    gint illegal_chars;             /* number of illegal chars */
+   gint illegal_chars_spc;         /* number of illegal space chars */
 };
 
 
@@ -134,8 +137,9 @@ void a_Url_set_data(DilloUrl *u, gchar *data);
 void a_Url_set_alt(DilloUrl *u, const gchar *alt);
 void a_Url_set_pos(DilloUrl *u, gint32 posx, gint32 posy);
 void a_Url_set_ismap_coords(DilloUrl *u, gchar *coord_str);
-gchar *a_Url_parse_hex_path(const DilloUrl *u);
-gchar *a_Url_string_strip_delimiters(gchar *str);
+gchar *a_Url_decode_hex_str(const gchar *str);
+gchar *a_Url_encode_hex_str(const gchar *str);
+gchar *a_Url_string_strip_delimiters(const gchar *str);
 
 #ifdef __cplusplus
 }
